#!/usr/bin/env python3
"""
xglucid is a part of the glucid package and provides 
a qt Graphical interface to configure a Lucid 8824 
Analog/Digital Audio Converter RS232 interface over 
a serial port.

Copyright (C) 2017,2018  Daniel R Mechanic (dan.mechanic@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License ONLY.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtWidgets import (QApplication, QMainWindow, QWidget, QPushButton,
                             QToolTip, QMessageBox, QGridLayout, QVBoxLayout,
                             QCheckBox, QHBoxLayout, QSlider, QLabel,
                             QSplashScreen)
from PyQt5.QtCore import QCoreApplication, Qt
from PyQt5.QtGui import QIcon, QPixmap
import sys
import os
from time import sleep
from glucid import Glucid8824_UI
from glucid.glucid8824 import Glucid8824
import configparser
import serial.tools.list_ports as lsports

CLEAN_NONEX_DEVICES=True

class xglucid(QtWidgets.QMainWindow, Glucid8824_UI.Ui_MainWindow):
    """xglucid is just a class to call the Qt MainWindow"""
    def __init__(self):
        super(self.__class__, self).__init__()
        self.setupUi(self)
        # load the default device from conf file
        currentdevice = Glucid8824.get_device_from_cfg()
        self.SerialPortCombo.setCurrentText(currentdevice)
        self.connection_label.setText("rs232")


        # cleanup non-valid serial ports
        # from interface...
        # first build a list of the ports
        portlist = []
        allports = []

        #print("BEFORE: all devices:")
        for i in range(self.SerialPortCombo.count()):
            allports.append(self.SerialPortCombo.itemText(i))
        #print(allports)
                
        for i in lsports.comports():
            portlist.append(i.device)

        # pyserial doesn't recognize these
        # and we need them...
        portlist.append("/dev/tty.usbserial")

        # then remove the SerialPortCombo items that aren't there
        #print("On this system:")
        #print(portlist)
        #print("now stripping:")
        for i in allports:
            #print("examining: "+i)
            if i not in portlist:
                #print("removing: "+i)
                self.SerialPortCombo.removeItem(
                    self.SerialPortCombo.findText(i))

                
        
def main():
    """call QApplication"""
    app = QApplication(sys.argv)
    splashmap = QPixmap(":/newPrefix/glucidSplash.png")
    splash = QSplashScreen(splashmap, QtCore.Qt.WindowStaysOnTopHint)
    splash.show()
    splash.showMessage("<h1>Make sure dip switch 1 is down on your lucid</h1>")
    app.processEvents()
    timer = QtCore.QElapsedTimer()
    timer.start()

    while timer.elapsed() < 4000 :
        app.processEvents()


    form = xglucid()
    form.show()

    splash.finish(form)
    app.exec_()

# if we wish to run from CLI without package
if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
glucid8824.py defines a class `Glucid8824` which
implements an api to a Lucid 8824 Analog/Digital Audio Converter
RS232 interface over a serial port.

See glucid.py for a `main` and associated functions 
that provide a useful command line tool `glucid` to configure 
the Lucid 8824/.

Copyright (C) 2017,2018  Daniel R Mechanic (dan.mechanic@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License ONLY.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

__version__ = '0.4.0a'
__author__ = 'Daniel R Mechanic (dan.mechanic@gmail.com)'
CONFIGFILE='.glucid.cfg'

import logging
import configparser
import getopt
import sys
import os
import serial
from distutils.util import strtobool
from _ast import Or

class Glucid8824:
    """The glucid8824 class represents a single Lucid
    ADA8824 unit and provides methods to communicate with
    a Lucid ADA8824 via its RS232 Serial port over the Serial
    Port of a Computer.
    """
    # hex constants:
    # sysex start & end byte
    STARTBYTE = 'f0'
    ENDBYTE = 'f7'

    # Lucid Manufacturer ID & 8824ID
    LUCIDINCID = ['00', '00', '5e']

    # Lucid 8824 Specific Model ID
    MODELID = '58'

    # Available Commands and thier
    # decimal values
    COMMANDSET = {
            'GetMode': '60',
            'SetMode': '20',
            'GetSync': '61',
            'SetSync': '21',
            'GetOptSrc': '62',
            'SetOptSrc': '22',
            'GetAnalogSrc': '63',
            'SetAnalogSrc': '23',
            'GetAesSrc': '64',
            'SetAesSrc': '24',
            'GetAnalogGain': '70',
            'SetAnalogGain': '30',
    }

    # OPTIONS FOR FRONT METER
    METER = ('Analog In', 'Digital In', 'Analog Out', 'Digital Out')

    # OPTION FOR DIGITAL IN 1/2
    DIG1 = ('AES', 'S/PDIF')

    # SYNC OPTIONS
    SYNC = (
        # 0:
        'ADAT',
        # 1:
        'WordClock',
        # 2:
        '44.1 Internal',
        # 3:
        '48 Internal',
        # 4:
        'AES In1',
        # 5:
        'AES In2',
        # 6:
        'AES In3',
        # 7:
        'S/PDIF In'
             )

    # OPTIONS FOR ANALOG OUT SRC
    ANALOG_SRC = (
        # 0:
        'ADAT In',
        # 1:
        'AES In'
             )

    # OPTIONS FOR AES DIGITAL OUT SRC
    AES_SRC = (
        #  0
        'ADAT In',
        # 1
        'Analog In'
        )

    # OPTIONS FOR ADAT OUT SRC
    OPTICAL_SRC = (
        # 0 analog input
        'Analog In',
        # 1 AES input
        'AES In'
    )

    # MIN DB LEVEL - 0
    MINIO = '00'

    # MAX DB LEVEL - 80
    MAXIO = '7f'

    # PRO (+4dBu) CONSTANTS
    # -8 dB IN
    # +1 dB OUT
    PLUS4IN = '58'
    PLUS4OUT = '61'

    # CONSUMER (-10dBV) CONSTANTS
    # +4 dB IN
    # -11 dB OUT
    MINUS10IN = '64'
    MINUS10OUT = '55'

    # Keep a dictionary of valid
    # devices to talk over, and the format
    # they use...
    # rs232 uses 9600
    # midi uses 31250

    DEVICES = {
        'rs232' : [
            '/dev/ttyUSB0',
            '/dev/tty.usbserial',
            'COM1',
            '/dev/ttyUSB1',
            '/dev/ttyUSB2',
            '/dev/ttyUSB3',
            '/dev/ttyUSB4',
            '/dev/ttyUSB5',
            '/dev/ttyS0',
            '/dev/ttyS1',
            '/dev/ttyS2',
            '/dev/ttyS3',
            '/dev/ttyS4',
            '/dev/ttyS5',
            '/dev/ttyAMA0',
            '/dev/ttyAMA1',
            '/dev/ttyACM0',
            '/dev/ttyACM1',
            '/dev/rfcomm0',
            '/dev/rfcomm1',
            '/dev/ircomm0',
            '/dev/ircomm1',
            '/dev/cuau0',
            '/dev/cuau1',
            '/dev/cuau2',
            '/dev/cuaU0',
            '/dev/cuaU1',
            'COM1',
            'COM2',
            'COM3',
            'COM4',
            '/dev/ttyUSB0',
            '/dev/ttyUSB1',
            '/dev/ttyUSB2',
            '/dev/ttyUSB3',
            '/dev/ttyUSB4',
            '/dev/ttyUSB4',
            '/dev/ttyUSB5',
            '/dev/ttyS0',
            '/dev/ttyS1',
            '/dev/ttyS2',
            '/dev/ttyS3',
            '/dev/ttyS4',
            '/dev/ttyS5',
            '/dev/ttyAMA0',
            '/dev/ttyAMA1',
            '/dev/ttyACM0',
            '/dev/ttyACM1',
            '/dev/rfcomm0',
            '/dev/ircomm0',
            '/dev/cuau0'
        ],
        'midi': [
            '/dev/midi1',
            '/dev/snd/midiC1D0'
        ]



    }

    def __init__(self,
                 LucidID='00', siface="/dev/ttyUSB0", tout=1):
        """Define required data structures and serial interface

           siface  - serial interface, defaults to /dev/ttyUSB0
           LucidID - The ID target of the Unit, defaults to 00
           tout    - serial timeout, in seconds, defaults to 1
        """
        self.ANALOG_GAIN = []

        # our 'config'
        self.glucidconf = configparser.ConfigParser()
        self.glucidconf.read(os.path.join(os.path.expanduser('~'), CONFIGFILE))
        # our serial connection
        # and default timeout
        self.conn = False
        self.tout = tout

        # our serial interface
        self.siface = siface

        # our baud rate
        self.baud=9600

        # InstanceID or LucidID 0-7
        self.INSTANCEID = LucidID

        # we either haven't connected
        # or had a failure
        self.error_flag = True

        self.curr_dig1val = 1
        
        # we had a device mismatch
        self.device_mismatch = False

        # our local representation of analog channel gain
        # 16 ... 8 ins then 8 outs
        # we store these values internally as integers in the
        # range that the lucid understands
        # we return strings representing db (-96)
        # we convert to hex when writing to Lucid
        self.gainlist = []

    def write_configfile(self):
        """Write the glucidconf to config file"""
        newconfig = open(os.path.join(os.path.expanduser('~'), CONFIGFILE), 'w')
        logging.info("Writing config to  %s" % os.path.join(os.path.expanduser('~'), CONFIGFILE))
        self.glucidconf.write(newconfig)
        newconfig.close()


        
    def set_device_from_configfile(self):
        # load config from file
        if 'Device' in self.glucidconf['DEFAULT']:
            self.siface = self.glucidconf['DEFAULT']['Device']
            logging.info("Read default %s from configfile" % self.siface)

    def is_in_linked(self):
        if 'LINKINCH' in self.glucidconf['DEFAULT']:
            retval = int(self.glucidconf['DEFAULT']['LINKINCH'])
        else:
            retval = 0
        return retval

    def is_out_linked(self):
        if 'LINKOUTCH' in self.glucidconf['DEFAULT']:
            retval = int(self.glucidconf['DEFAULT']['LINKOUTCH'])
        else:
            retval = 0
        return retval

    def get_aes_source(self, RETSTRING=True):
        """Send the GetAesSrc command and
        Get the currently set source of AES
        Digital Output.  Return
        String representation of result.

        If RETSTRING is FALSE return the raw value
        """
        DEFAULT_AES_SRC = 0 # Analog In
        retval = self.sendCommand('GetAesSrc')

        if not retval:
            raise ValueError("Bad Data Received")
            retval = 0
        else:
            retval = retval[0]

        if RETSTRING:
            return self.AES_SRC[retval]
        else:
            return retval

    def set_aes_src(self, srcval=False):
        """Set the source of the AES Digital
        OUTPUT.

        srcval - 0 : ADAT IN
                 1 : Analog IN
        """

        if int(srcval) < 0 or int(srcval) > 1:
            logging.error("set_aes_source value %s invalid!" % srcval)
            return False
        logging.info("set_aes_src:setting to %s" % self.AES_SRC[srcval])
        self.glucidconf['DEFAULT']['AES_SRC'] = str(srcval)
        self.sendCommand('SetAesSrc', [self.int_to_hex(srcval)])

        
    def get_instanceid(self):
        """return the currently set instanceid without
           retrieving it first, instanceid defaults to 00
           but is updated each time the Unit is read from"""
        return self.INSTANCEID

    def get_opt_source(self, RETSTRING=True):
        """Get the source of the ADAT Digit
        Input by sending the GetOptSrc command
        """
        DEFAULT_OPTICAL_SRC = 0 # Analog In

        retval = self.sendCommand('GetOptSrc')
        retval = retval[0]
            
        if RETSTRING:
            return self.OPTICAL_SRC[retval]
        else:
            return retval

    def set_opt_src(self, srcval=False):
        """Set the source of the ADAT Digital
        OUTPUT.

        srcval - 0 : Analog IN
                 1 : AES IN
        """

        if int(srcval) < 0 or int(srcval) > 1:
            logging.error("set_source value %s invalid!" % srcval)
            return False
        logging.info("set_opt_src: setting to %s" %
                     self.OPTICAL_SRC[srcval])
        self.glucidconf['DEFAULT']['OPTICAL_SRC'] = str(srcval)
        self.sendCommand('SetOptSrc', [self.int_to_hex(srcval)])

    def get_analog_source(self, RETSTRING=True):
        DEFAULT_ANALOG_SRC = 0 # ADAT In

        retval = self.sendCommand('GetAnalogSrc')
        retval = retval[0]
            
        if RETSTRING:
            return self.ANALOG_SRC[retval]
        else:
            return retval

    def set_analog_src(self, srcval=False):
        """Set the source of the Analog OUTPUT.

        srcval - 0 : ADAT IN
                 1 : AES IN
        """

        if int(srcval) < 0 or int(srcval) > 1:
            logging.error("set_source value %s invalid!" % srcval)
            return False
        logging.info("set_analog_src: setting to %s" %
                     self.ANALOG_SRC[srcval])
        self.glucidconf['DEFAULT']['ANALOG_SRC'] = str(srcval)
        self.sendCommand('SetAnalogSrc', [self.int_to_hex(srcval)])

    def get_sync_source(self, RETSTRING=True):
        """Get the source for clock sync and return
        a String representation

        If RETSTRING is FALSE, return the raw value
        """
        DEFAULT_SYNC = 1 # Wordclock
        
        retval = self.sendCommand('GetSync')
        if isinstance(retval, list):
            retval = retval[0]
        else:
            raise ValueError("Bad Data Recieved")
        retval = 0

            
        if RETSTRING:
            return self.SYNC[retval]
        else:
            return retval

    def set_sync_source(self, syncsrc=False, RETSTRING=True):
        """Set the source for clock sync

        syncsrc - 0 : ADAT
                  1 : WordClock
                  2 : INTERNAL 44.1
                  3 : INTERNAL 48
                  4 : AES 1
                  5 : AES 2
                  6 : AES 3
                  7 : S/PDIF
        """
        if (syncsrc < 0 or syncsrc > 7) and syncsrc not in self.SYNC:
            logging.error("set_sync_source value %s invalid!" % syncsrc)
            return False
        synchex = '{:02}'.format(syncsrc, 'x')
        logging.info("set_sync_source: set to %s" % self.SYNC[syncsrc])
        logging.info("set_sync_source: set to %s" % synchex)
        self.glucidconf['DEFAULT']['SYNC'] = str(synchex)
        self.sendCommand('SetSync', [synchex])


    def get_meter(self, RETSTRING=True):
        """Get the source for the front meters of the 8824
        """
        DEFAULT_METER = 2 # Analog Out
        
        retval = self.sendCommand('GetMode')
        retval = int("{0:02b}".format(retval[0])[-2:], 2)
            
        if RETSTRING:
            return self.METER[retval]
        else:
            return retval

    def set_meter_and_dig1(self, metersrc=False):
        """Set the source for front meters of the 8824
        AND to route S/SPIF

        metersrc - 0 : Meter Analog IN
                       AES DIG IN 1,2
                   1 : Digital IN
                       AES DIG IN 1,2
                   2 : Analog OUT
                       AES DIG IN 1,2
                   3 : Digital OUT
                       AES DIG IN 1,2
                   4 : Analog IN
                       S/PDIF DIG IN 1,2
                   5 : Digital IN
                       S/PDIF DIG IN 1,2
                   6 : Analog OUT
                       S/PDIF DIG IN 1,2
                   7 : Digital OUT
        """

        if int(metersrc) < 0 or int(metersrc) > 7:
            logging.error("set_meter_source val %s invalid!" % metersrc)
            return False

        modeval = int(metersrc)
        modeval = self.int_to_hex(modeval)
        logging.info("set_meter_and_dig1: setting MODE to %s" % modeval)
        if int(modeval) < 4:
            self.glucidconf['DEFAULT']['DIG1'] = '0'
            tmpval = modeval
        else:
            tmpval = int(modeval)-4
            self.glucidconf['DEFAULT']['DIG1'] = '1'
        self.glucidconf['DEFAULT']['METER']=str(tmpval)
        self.sendCommand('SetMode', [modeval])

    def set_meter(self, metersrc=False):
        """Set the source for front meters of the 8824

        metersrc - 0 : Analog IN
                   1 : Digital IN
                   2 : Analog OUT
                   3 : Digital OUT
        """

        if int(metersrc) < 0 or int(metersrc) > 3:
            logging.error("set_meter_source val %s invalid!" % metersrc)
            return False
        logging.info("set_meter: setting to %s" % self.METER[metersrc])

        curr_dig1 = self.get_dig1(False)
            
        logging.info("set_meter: Dig1 is %s" % curr_dig1)

        modeval = (int(curr_dig1)*4)+int(metersrc)
        modeval = self.int_to_hex(modeval)
        logging.info("set_meter: setting MODE to %s" % modeval)
        self.glucidconf['DEFAULT']['METER']=str(metersrc)
        self.sendCommand('SetMode', [modeval])

    def get_dig1(self, RETSTRING=True):
        """Get the source of Digital Channels 1,2
        Input (AES of SPDIF) by sending the GetMode command
        """
        DEFAULT_DIG1 = 1 # SPDIF
        
        retval = self.sendCommand('GetMode')
        retval = int("{0:04b}".format(retval[0])[-3], 2)            

        if RETSTRING:
            return self.DIG1[retval]
        else:
            return retval
        
    def set_dig1(self, srcval=False):
        """Set the source for Digital Channels 1,2:

        srcval - 0 : AES
                 1 : S/PDIF
        """

        if int(srcval) < 0 or int(srcval) > 1:
            logging.error("set_dig1 value %s invalid!" % srcval)
            return False
        logging.info("set_dig1: setting to %s" % self.DIG1[srcval])
        curr_meter = self.get_meter(False)
        logging.info("set_dig1: meter is %s" % curr_meter)

        modeval = (int(srcval)*4)+int(curr_meter)
        modeval = self.int_to_hex(modeval)
        logging.info("set_meter: setting MODE to %s" % modeval)
        self.glucidconf['DEFAULT']['DIG1']=str(srcval)
        self.sendCommand('SetMode', [modeval])

    @staticmethod
    def gain_int_to_db_string(intval):
        """Substract 96 and return the signed value as a string
        """
        if (intval-96) >= 0:
            return "+"+str(intval-96)
        return str(intval-96)

    @staticmethod
    def int_to_hex(intval):
        """A convenience function, use format to convert int to hex"""
        if type(intval) == "string":
            intval = int(intval)
        return '{:02}'.format(intval, 'x')

    @staticmethod
    def get_device_from_cfg():
        """Read the default device stored in our config file"""
        tmpconfig = configparser.ConfigParser()
        tmpconfig.read(os.path.join(os.path.expanduser('~'), CONFIGFILE))
        if 'Device' in tmpconfig['DEFAULT']:
            return tmpconfig['DEFAULT']['Device']
        else:
            return '/dev/ttyUSB0'


    def get_gain(self):
        """Populates the internal datastructure `gainlist` by
        sending the `GetAnalogGain` command to the 8824
        """
        # this returns a list of strings
        # representing dB
        # it also updates the gainlist

        # lucidgainlist... get from Lucid
        DEFAULT_GAINLIST = '88,88,88,88,88,88,88,88,97,97,97,97,97,97,97,97'

        retlist = []
        self.gainlist = []
        
        lgainlist = self.sendCommand('GetAnalogGain')
        self.gainlist = []
        self.log_gain_list(lgainlist)

        # the lucid reverses this 8..1 8..1
        # which we didn't know until too late
        # so we fix this here:
        self.gainlist = lgainlist[7::-1] + lgainlist[15:7:-1]
        
        for i in range(0, len(self.gainlist)):
            retlist.append(
                Glucid8824.gain_int_to_db_string(int(self.gainlist[i])))

        logging.info("Returning Gainlist")
        logging.info(retlist)
        return retlist

    def log_gain_list(self, gainlist=False):
        """Log the current gainlist, for testing purposes
        """
        if not gainlist:
            return False

        for g in self.gainlist:
            logging.info("%s" % Glucid8824.gain_int_to_db_string(int(g)))

    def update_channel_in_gainlist(self, gain=False, channel=False):
        """Update a single channel in the gain list, must have
        called get_gain to get the list from the 8824 first.

        gain    - value to set gain to -95 to +32
        channel - channel to set
        """
        logging.info("update_channel_in_gainlist: %s to %s" %
                     (channel, gain))

        channel = int(channel)
        # makes sure gain and channel is set
        if (channel < 0 or channel > 15):
            logging.error("update_channel_in_gainlist: bad channel value")
            return False

        # need to read all channels before writing one
        if (len(self.gainlist) < 15):
            logging.error("update_channel_in_gainlist: call get_gain first")
            return False

        # Replace the one channel gain
        logging.info("update_channel_in_gainlist: Current gainlist:")
        self.log_gain_list(self.gainlist)

        self.gainlist[channel] = int(gain)+96

        # set back the new gain
        logging.info("set_gain: New gainlist:")
        self.log_gain_list(self.gainlist)

    def set_gainlist_all_stage(self, gain=-99, input_or_output=0):
        """Set all input or output channels to the same gain value

        gain - integer -95 to 32

        input_or_output -  0: INPUT
                           1: OUTPUT
        """

        # input=0 ; output=1
        # gain is a string -95 +32
        if int(gain) < -95 or int(gain) > 32:
            logging.error("set_gainlist_all_stage: out of range %s" %
                          gain)
            return False

        gain = int(gain)
        logging.info("set_gainlist_all_stage: % , %s" % (gain, channel))

        if (input_or_output != 0 and input_or_output != 1):
            logging.error("set_gainlist_all_stage:channel not 0 or 1 :%s" %
                          channel)
            return False

        if (len(self.gainlist) < 15):
            logging.error("set_gainlist_all_stage: call get_gain first")
            return False

        # Replace half the gain
        logging.info("set_gainlist_all_stage: Current gainlist:")
        log_gain_list(self.gainlist)

        chstart = 8 * input_or_output
        for channel in range(chstart, chstart + 8):
            self.gainlist[channel] = int(gain) + 96
            logging.info("set_gainlist_all_stage: set %s to %d" %
                         (channel, int(gain)+96))

        # set back the new gain
        logging.info("set_gainlist_all_stage: New gainlist:")
        for g in self.gainlist:
            logging.info("%s" % Glucid8824.gain_int_to_db_string(g))

    def write_gainlist_to_lucid(self):
        """Convert `gainlist` to hex and call `SetAnalogGain`
        to write the values to the 8824
        """
        # we have to convert our local gainlist to hex
        # and use sendCommand to write it.
        # first we need to put the channels
        # in the ordering the lucid likes:
        # 8..1 8..1
        lucidgainlist = [];
        lucidgainlist = self.gainlist[7::-1] + self.gainlist[15:7:-1]

        cmdArg = []
        for chgain in lucidgainlist:
            chgain = hex(int(chgain))[2:]
            cmdArg.append(chgain)
            logging.info("write_gainlist_to_lucid: add %s to cmdArg" %
                         (chgain))

        logging.info("write_gainlist_to_lucid: calling sendCommand")
        
        self.sendCommand('SetAnalogGain', cmdArg)

    ###########################
    # Begin lower-level methods
    ###########################

    def sendCommand(self, cmdkey, cmdArg=['00'], checkcommand=True):
        # if checkcommand is False we can use values outside
        # of our command dictionary.  Good for looking for undocumented
        # commands
        if checkcommand and cmdkey not in self.COMMANDSET:
            logging.error("SendCommand: Invalid Command Key: %s", cmdkey)
            return False

        logging.info('sendCommand sending %s:%s',
                     cmdkey,
                     self.COMMANDSET[cmdkey])
        if self.error_flag:
            logging.error("SendCommand: Error Flag - aborting command")
            return False

        cmd = ""
        cmd += self.STARTBYTE+' '
        for hexcode in self.LUCIDINCID:
            cmd += hexcode+' '
        cmd += self.MODELID+' '
        cmd += '{:02}'.format(int(self.INSTANCEID))+' '
        cmd += self.COMMANDSET[cmdkey]+' '
        for cmdargs in cmdArg:
            cmd += cmdargs+' '
        cmd += self.ENDBYTE
        logging.info('sendCommand: Sending \"%s\"', cmd)

        self.conn.write(bytes.fromhex(cmd))

        if self.siface in self.DEVICES['rs232'] :
            logging.info("siface is %s"%self.siface)
            hexdata = self.getLinefromSerial()
            if not hexdata:
                logging.error("Did NOT receive data response.")
                return False
            return self.getResponse(cmdkey, hexdata)
        return True
    
    def getResponse(self, cmdkey, hexdata):
        """Parse the hex data received, set error flags
        if the repsponse is poorly formed
        """

        logging.info("getResponse: Length of hexdata is %s", len(hexdata))
        # if logging print out our response
        for i in range(0, len(hexdata)):
            logging.info("hexdata[%s]:%s", i, format(hexdata[i], '#04x'))

        retdata = []
        if self.error_flag:
            print("Error Flag is set, not attempting response")
            return False

        if not self.isValidHexData(hexdata):
            logging.error("getReponse received Error from isValidHexData")
            return False

        # get the beginning of the command
        StartIndex = hexdata.index(int(self.STARTBYTE, 16))

        # we are getting a response
        if hexdata[StartIndex+6] != int('0x05', 16):
            logging.error("getLucidResponse looking for OxO5:%s",
                          hexdata[StartIndex+6])
            return False

        # expect response to be same as command issued
        # clean this up?  all cases been checked?
            logging.error("failed at verifying commandkey %s:%s ",
                          self.COMMANDSET[cmdkey], hexdata[StartIndex+7])
            return False

        retindex = StartIndex + 8
        while ((retindex < len(hexdata))
                and (hexdata[retindex] != int(self.ENDBYTE, 16))):
            logging.info("appending %s to response string",
                         hexdata[retindex])
            retdata.append(hexdata[retindex])
            retindex += 1
        return retdata

    def isValidHexData(self, hexdata):
        """Check for a well formed instruction"""
        # Have a Valid Start Byte?
        try:
            StartIndex = hexdata.index(int(self.STARTBYTE, 16))
        except ValueError:
            logging.error("isValidHexData failed to find StartIndex")
            return False
        # Have a Valid End Byte?
        try:
            hexdata.index(int(self.ENDBYTE, 16))
        except ValueError:
            logging.error("isValidHexData failed at finding ENDBYTE")
            return False

        # Have a Valid Manufacturer Id?
        for i in range(3):
            if hexdata[StartIndex+1+i] != int(self.LUCIDINCID[i], 16):
                logging.error("isValidHexData:%s failed manufacturer id",
                              StartIndex+1+i)
                return False

        # Have a Valid ModelId?
        if hexdata[StartIndex+4] != int(self.MODELID, 16):
            logging.error("isValidHexData:%s failed at modelId",
                          StartIndex+1+i)
            return False

        # Read and Update InstanceID should be last.
        # If we are receiving the wrong InstanceID
        # set the INSTANCEIDDIRTY flag
        if hexdata[StartIndex+5] != int(self.INSTANCEID, 16):
            logging.warning("isValidHexData: unexpected InstanceId - %s",
                            hexdata[StartIndex+5])
            logging.warning("isValidHexData Updating from %s to %s",
                            self.InstanceId, hexdata[StartIndex+5])
            self.INSTANCEID = hexdata[StartIndex+5]
            self.device_mismatch = True
            return False
        else:
            # clear device mismatch flag if set
            self.device_mismatch = False
        return True

    def connect(self):
        """Attempt to open a serial connection to self.siface
        """
        logging.info("LucidConnection running connect")

        try:
            logging.info("LucidConnection opening rs232 device %s"%self.siface)
            self.conn = serial.Serial(self.siface, baudrate=self.baud, timeout=self.tout)
        except serial.SerialException as se:
            logging.error("Could not open connection %s", self.siface)
            self.error_flag = True
            return False
        except Exception as e:
            logging.error("Could not open connection %s", self.siface)
            self.error_flag = True
            return False
        else:
            self.error_flag = False

            return True
        

    def disconnect(self):
        """Close serial connection"""
        try:
            self.conn.close()
        except serial.SerialException as se:
            self.error_flag = True
            logging.error("Could not close serial connection %s", self.siface)

    def get_iface(self):
        """Return the currently set serial interface"""
        return self.siface

    def clear_errorflag(self):
        """set error_flag and device_mismatch to False"""
        self.error_flag = False
        self.device_mismatch = False

    def set_errorflag(self):
        self.error_flag = True

    def getLinefromSerial(self):
        """Returns a line of data from the serial connection"""
        if self.error_flag:
            logging.warning("getLinefromSerial not connected")
            self.set_errorflag()
            return False

        # we try 10 times
        for i in range(0, 3):
            myline = self.conn.readline()
            logging.info("Received %s bytes", len(myline))
            if len(myline) > 0:
                break
            if i == 9:
                return False
        return myline



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './Glucid8824.ui'
#
# Created by: PyQt5 UI code generator 5.11.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.setEnabled(True)
        MainWindow.resize(901, 473)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QtCore.QSize(901, 470))
        MainWindow.setMaximumSize(QtCore.QSize(901, 516))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/newPrefix/lucid8824_icon.bmp"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon.addPixmap(QtGui.QPixmap(":/newPrefix/lucid8824_icon.bmp"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        icon.addPixmap(QtGui.QPixmap(":/newPrefix/lucid8824_icon.bmp"), QtGui.QIcon.Disabled, QtGui.QIcon.Off)
        icon.addPixmap(QtGui.QPixmap(":/newPrefix/lucid8824_icon.bmp"), QtGui.QIcon.Disabled, QtGui.QIcon.On)
        icon.addPixmap(QtGui.QPixmap(":/newPrefix/lucid8824_icon.bmp"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        icon.addPixmap(QtGui.QPixmap(":/newPrefix/lucid8824_icon.bmp"), QtGui.QIcon.Active, QtGui.QIcon.On)
        icon.addPixmap(QtGui.QPixmap(":/newPrefix/lucid8824_icon.bmp"), QtGui.QIcon.Selected, QtGui.QIcon.Off)
        icon.addPixmap(QtGui.QPixmap(":/newPrefix/lucid8824_icon.bmp"), QtGui.QIcon.Selected, QtGui.QIcon.On)
        MainWindow.setWindowIcon(icon)
        self.centralwidget = xglucidWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setEnabled(False)
        self.label.setGeometry(QtCore.QRect(-1, 0, 901, 471))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setMaximumSize(QtCore.QSize(901, 471))
        self.label.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.label.setStyleSheet("background-image: url(:/newPrefix/glucidLayout3-nosig.png);")
        self.label.setMidLineWidth(1)
        self.label.setText("")
        self.label.setObjectName("label")
        self.SliderIN_1 = QtWidgets.QSlider(self.centralwidget)
        self.SliderIN_1.setEnabled(False)
        self.SliderIN_1.setGeometry(QtCore.QRect(286, 73, 21, 138))
        self.SliderIN_1.setAutoFillBackground(False)
        self.SliderIN_1.setMinimum(-95)
        self.SliderIN_1.setMaximum(31)
        self.SliderIN_1.setProperty("value", 0)
        self.SliderIN_1.setSliderPosition(0)
        self.SliderIN_1.setOrientation(QtCore.Qt.Vertical)
        self.SliderIN_1.setObjectName("SliderIN_1")
        self.SliderIN_2 = QtWidgets.QSlider(self.centralwidget)
        self.SliderIN_2.setEnabled(False)
        self.SliderIN_2.setGeometry(QtCore.QRect(342, 73, 21, 138))
        self.SliderIN_2.setMinimum(-95)
        self.SliderIN_2.setMaximum(31)
        self.SliderIN_2.setProperty("value", 0)
        self.SliderIN_2.setSliderPosition(0)
        self.SliderIN_2.setOrientation(QtCore.Qt.Vertical)
        self.SliderIN_2.setObjectName("SliderIN_2")
        self.SliderIN_3 = QtWidgets.QSlider(self.centralwidget)
        self.SliderIN_3.setEnabled(False)
        self.SliderIN_3.setGeometry(QtCore.QRect(397, 73, 21, 138))
        self.SliderIN_3.setMinimum(-95)
        self.SliderIN_3.setMaximum(31)
        self.SliderIN_3.setProperty("value", 0)
        self.SliderIN_3.setSliderPosition(0)
        self.SliderIN_3.setOrientation(QtCore.Qt.Vertical)
        self.SliderIN_3.setObjectName("SliderIN_3")
        self.SliderIN_6 = QtWidgets.QSlider(self.centralwidget)
        self.SliderIN_6.setEnabled(False)
        self.SliderIN_6.setGeometry(QtCore.QRect(560, 73, 21, 138))
        self.SliderIN_6.setMinimum(-95)
        self.SliderIN_6.setMaximum(31)
        self.SliderIN_6.setProperty("value", 0)
        self.SliderIN_6.setSliderPosition(0)
        self.SliderIN_6.setOrientation(QtCore.Qt.Vertical)
        self.SliderIN_6.setObjectName("SliderIN_6")
        self.SliderIN_4 = QtWidgets.QSlider(self.centralwidget)
        self.SliderIN_4.setEnabled(False)
        self.SliderIN_4.setGeometry(QtCore.QRect(452, 73, 21, 138))
        self.SliderIN_4.setMinimum(-95)
        self.SliderIN_4.setMaximum(31)
        self.SliderIN_4.setProperty("value", 0)
        self.SliderIN_4.setSliderPosition(0)
        self.SliderIN_4.setOrientation(QtCore.Qt.Vertical)
        self.SliderIN_4.setObjectName("SliderIN_4")
        self.SliderIN_5 = QtWidgets.QSlider(self.centralwidget)
        self.SliderIN_5.setEnabled(False)
        self.SliderIN_5.setGeometry(QtCore.QRect(506, 73, 21, 138))
        self.SliderIN_5.setMinimum(-95)
        self.SliderIN_5.setMaximum(31)
        self.SliderIN_5.setProperty("value", 0)
        self.SliderIN_5.setSliderPosition(0)
        self.SliderIN_5.setOrientation(QtCore.Qt.Vertical)
        self.SliderIN_5.setObjectName("SliderIN_5")
        self.SliderIN_7 = QtWidgets.QSlider(self.centralwidget)
        self.SliderIN_7.setEnabled(False)
        self.SliderIN_7.setGeometry(QtCore.QRect(616, 73, 21, 138))
        self.SliderIN_7.setMinimum(-95)
        self.SliderIN_7.setMaximum(31)
        self.SliderIN_7.setProperty("value", 0)
        self.SliderIN_7.setSliderPosition(0)
        self.SliderIN_7.setOrientation(QtCore.Qt.Vertical)
        self.SliderIN_7.setObjectName("SliderIN_7")
        self.SliderIN_8 = QtWidgets.QSlider(self.centralwidget)
        self.SliderIN_8.setEnabled(False)
        self.SliderIN_8.setGeometry(QtCore.QRect(671, 73, 21, 138))
        self.SliderIN_8.setMinimum(-95)
        self.SliderIN_8.setMaximum(31)
        self.SliderIN_8.setProperty("value", 0)
        self.SliderIN_8.setSliderPosition(0)
        self.SliderIN_8.setOrientation(QtCore.Qt.Vertical)
        self.SliderIN_8.setObjectName("SliderIN_8")
        self.SliderOUT_3 = QtWidgets.QSlider(self.centralwidget)
        self.SliderOUT_3.setEnabled(False)
        self.SliderOUT_3.setGeometry(QtCore.QRect(397, 275, 20, 138))
        self.SliderOUT_3.setMinimum(-95)
        self.SliderOUT_3.setMaximum(31)
        self.SliderOUT_3.setProperty("value", 0)
        self.SliderOUT_3.setOrientation(QtCore.Qt.Vertical)
        self.SliderOUT_3.setObjectName("SliderOUT_3")
        self.SliderOUT_1 = QtWidgets.QSlider(self.centralwidget)
        self.SliderOUT_1.setEnabled(False)
        self.SliderOUT_1.setGeometry(QtCore.QRect(287, 275, 21, 138))
        self.SliderOUT_1.setMinimum(-95)
        self.SliderOUT_1.setMaximum(31)
        self.SliderOUT_1.setProperty("value", 0)
        self.SliderOUT_1.setOrientation(QtCore.Qt.Vertical)
        self.SliderOUT_1.setInvertedAppearance(False)
        self.SliderOUT_1.setObjectName("SliderOUT_1")
        self.SliderOUT_8 = QtWidgets.QSlider(self.centralwidget)
        self.SliderOUT_8.setEnabled(False)
        self.SliderOUT_8.setGeometry(QtCore.QRect(672, 275, 21, 138))
        self.SliderOUT_8.setMinimum(-95)
        self.SliderOUT_8.setMaximum(31)
        self.SliderOUT_8.setProperty("value", 0)
        self.SliderOUT_8.setOrientation(QtCore.Qt.Vertical)
        self.SliderOUT_8.setObjectName("SliderOUT_8")
        self.SliderOUT_5 = QtWidgets.QSlider(self.centralwidget)
        self.SliderOUT_5.setEnabled(False)
        self.SliderOUT_5.setGeometry(QtCore.QRect(507, 275, 21, 138))
        self.SliderOUT_5.setMinimum(-95)
        self.SliderOUT_5.setMaximum(31)
        self.SliderOUT_5.setProperty("value", 0)
        self.SliderOUT_5.setOrientation(QtCore.Qt.Vertical)
        self.SliderOUT_5.setObjectName("SliderOUT_5")
        self.SliderOUT_7 = QtWidgets.QSlider(self.centralwidget)
        self.SliderOUT_7.setEnabled(False)
        self.SliderOUT_7.setGeometry(QtCore.QRect(617, 275, 21, 138))
        self.SliderOUT_7.setMinimum(-95)
        self.SliderOUT_7.setMaximum(31)
        self.SliderOUT_7.setProperty("value", 0)
        self.SliderOUT_7.setOrientation(QtCore.Qt.Vertical)
        self.SliderOUT_7.setObjectName("SliderOUT_7")
        self.SliderOUT_6 = QtWidgets.QSlider(self.centralwidget)
        self.SliderOUT_6.setEnabled(False)
        self.SliderOUT_6.setGeometry(QtCore.QRect(563, 275, 21, 138))
        self.SliderOUT_6.setMinimum(-95)
        self.SliderOUT_6.setMaximum(31)
        self.SliderOUT_6.setProperty("value", 0)
        self.SliderOUT_6.setOrientation(QtCore.Qt.Vertical)
        self.SliderOUT_6.setObjectName("SliderOUT_6")
        self.SliderOUT_2 = QtWidgets.QSlider(self.centralwidget)
        self.SliderOUT_2.setEnabled(False)
        self.SliderOUT_2.setGeometry(QtCore.QRect(342, 275, 21, 138))
        self.SliderOUT_2.setMinimum(-95)
        self.SliderOUT_2.setMaximum(31)
        self.SliderOUT_2.setProperty("value", 0)
        self.SliderOUT_2.setOrientation(QtCore.Qt.Vertical)
        self.SliderOUT_2.setObjectName("SliderOUT_2")
        self.SliderOUT_4 = QtWidgets.QSlider(self.centralwidget)
        self.SliderOUT_4.setEnabled(False)
        self.SliderOUT_4.setGeometry(QtCore.QRect(453, 275, 21, 138))
        self.SliderOUT_4.setMinimum(-95)
        self.SliderOUT_4.setMaximum(31)
        self.SliderOUT_4.setProperty("value", 0)
        self.SliderOUT_4.setOrientation(QtCore.Qt.Vertical)
        self.SliderOUT_4.setObjectName("SliderOUT_4")
        self.LucidMeterCombo = QtWidgets.QComboBox(self.centralwidget)
        self.LucidMeterCombo.setEnabled(False)
        self.LucidMeterCombo.setGeometry(QtCore.QRect(64, 93, 72, 20))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(4)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.LucidMeterCombo.sizePolicy().hasHeightForWidth())
        self.LucidMeterCombo.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Source Sans Pro Light")
        font.setPointSize(7)
        font.setKerning(True)
        self.LucidMeterCombo.setFont(font)
        self.LucidMeterCombo.setEditable(False)
        self.LucidMeterCombo.setCurrentText("Analog IN")
        self.LucidMeterCombo.setIconSize(QtCore.QSize(0, 0))
        self.LucidMeterCombo.setFrame(False)
        self.LucidMeterCombo.setObjectName("LucidMeterCombo")
        self.LucidMeterCombo.addItem("")
        self.LucidMeterCombo.addItem("")
        self.LucidMeterCombo.addItem("")
        self.LucidMeterCombo.addItem("")
        self.LucidSyncCombo = QtWidgets.QComboBox(self.centralwidget)
        self.LucidSyncCombo.setEnabled(False)
        self.LucidSyncCombo.setGeometry(QtCore.QRect(64, 165, 72, 20))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(4)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.LucidSyncCombo.sizePolicy().hasHeightForWidth())
        self.LucidSyncCombo.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Source Sans Pro Light")
        font.setPointSize(7)
        font.setKerning(True)
        self.LucidSyncCombo.setFont(font)
        self.LucidSyncCombo.setEditable(False)
        self.LucidSyncCombo.setCurrentText("ADAT")
        self.LucidSyncCombo.setIconSize(QtCore.QSize(0, 0))
        self.LucidSyncCombo.setFrame(False)
        self.LucidSyncCombo.setObjectName("LucidSyncCombo")
        self.LucidSyncCombo.addItem("")
        self.LucidSyncCombo.addItem("")
        self.LucidSyncCombo.addItem("")
        self.LucidSyncCombo.addItem("")
        self.LucidSyncCombo.addItem("")
        self.LucidSyncCombo.addItem("")
        self.LucidSyncCombo.addItem("")
        self.LucidSyncCombo.addItem("")
        self.LucidIdCombo = QtWidgets.QComboBox(self.centralwidget)
        self.LucidIdCombo.setEnabled(True)
        self.LucidIdCombo.setGeometry(QtCore.QRect(113, 298, 31, 22))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(4)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.LucidIdCombo.sizePolicy().hasHeightForWidth())
        self.LucidIdCombo.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Source Sans Pro Light")
        font.setPointSize(7)
        font.setKerning(True)
        self.LucidIdCombo.setFont(font)
        self.LucidIdCombo.setEditable(False)
        self.LucidIdCombo.setCurrentText("00")
        self.LucidIdCombo.setIconSize(QtCore.QSize(0, 0))
        self.LucidIdCombo.setFrame(False)
        self.LucidIdCombo.setObjectName("LucidIdCombo")
        self.LucidIdCombo.addItem("")
        self.LucidIdCombo.addItem("")
        self.LucidIdCombo.addItem("")
        self.LucidIdCombo.addItem("")
        self.LucidIdCombo.addItem("")
        self.LucidIdCombo.addItem("")
        self.LucidIdCombo.addItem("")
        self.LucidIdCombo.addItem("")
        self.LucidSpdifCombo = QtWidgets.QComboBox(self.centralwidget)
        self.LucidSpdifCombo.setEnabled(False)
        self.LucidSpdifCombo.setGeometry(QtCore.QRect(730, 369, 75, 20))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(4)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.LucidSpdifCombo.sizePolicy().hasHeightForWidth())
        self.LucidSpdifCombo.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Source Sans Pro Light")
        font.setPointSize(8)
        font.setKerning(True)
        self.LucidSpdifCombo.setFont(font)
        self.LucidSpdifCombo.setEditable(False)
        self.LucidSpdifCombo.setCurrentText("1,2 AES")
        self.LucidSpdifCombo.setIconSize(QtCore.QSize(0, 0))
        self.LucidSpdifCombo.setFrame(False)
        self.LucidSpdifCombo.setObjectName("LucidSpdifCombo")
        self.LucidSpdifCombo.addItem("")
        self.LucidSpdifCombo.addItem("")
        self.LucidAnalogSrcCombo = QtWidgets.QComboBox(self.centralwidget)
        self.LucidAnalogSrcCombo.setEnabled(False)
        self.LucidAnalogSrcCombo.setGeometry(QtCore.QRect(730, 295, 75, 20))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(4)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.LucidAnalogSrcCombo.sizePolicy().hasHeightForWidth())
        self.LucidAnalogSrcCombo.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Source Sans Pro Light")
        font.setPointSize(9)
        font.setKerning(True)
        self.LucidAnalogSrcCombo.setFont(font)
        self.LucidAnalogSrcCombo.setEditable(False)
        self.LucidAnalogSrcCombo.setCurrentText("ADAT IN")
        self.LucidAnalogSrcCombo.setIconSize(QtCore.QSize(0, 0))
        self.LucidAnalogSrcCombo.setFrame(False)
        self.LucidAnalogSrcCombo.setObjectName("LucidAnalogSrcCombo")
        self.LucidAnalogSrcCombo.addItem("")
        self.LucidAnalogSrcCombo.addItem("")
        self.LucidOpticalCombo = QtWidgets.QComboBox(self.centralwidget)
        self.LucidOpticalCombo.setEnabled(False)
        self.LucidOpticalCombo.setGeometry(QtCore.QRect(730, 166, 75, 27))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(4)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.LucidOpticalCombo.sizePolicy().hasHeightForWidth())
        self.LucidOpticalCombo.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Source Sans Pro Light")
        font.setPointSize(9)
        font.setKerning(True)
        self.LucidOpticalCombo.setFont(font)
        self.LucidOpticalCombo.setEditable(False)
        self.LucidOpticalCombo.setCurrentText("Analog IN")
        self.LucidOpticalCombo.setIconSize(QtCore.QSize(0, 0))
        self.LucidOpticalCombo.setFrame(False)
        self.LucidOpticalCombo.setObjectName("LucidOpticalCombo")
        self.LucidOpticalCombo.addItem("")
        self.LucidOpticalCombo.addItem("")
        self.LucidAesCombo = QtWidgets.QComboBox(self.centralwidget)
        self.LucidAesCombo.setEnabled(False)
        self.LucidAesCombo.setGeometry(QtCore.QRect(730, 115, 75, 28))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(4)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.LucidAesCombo.sizePolicy().hasHeightForWidth())
        self.LucidAesCombo.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Source Sans Pro Light")
        font.setPointSize(9)
        font.setKerning(True)
        self.LucidAesCombo.setFont(font)
        self.LucidAesCombo.setEditable(False)
        self.LucidAesCombo.setCurrentText("ADAT IN")
        self.LucidAesCombo.setIconSize(QtCore.QSize(0, 0))
        self.LucidAesCombo.setFrame(False)
        self.LucidAesCombo.setObjectName("LucidAesCombo")
        self.LucidAesCombo.addItem("")
        self.LucidAesCombo.addItem("")
        self.ProINButton = QtWidgets.QPushButton(self.centralwidget)
        self.ProINButton.setEnabled(False)
        self.ProINButton.setGeometry(QtCore.QRect(180, 96, 70, 37))
        self.ProINButton.setObjectName("ProINButton")
        self.ConINButton = QtWidgets.QPushButton(self.centralwidget)
        self.ConINButton.setEnabled(False)
        self.ConINButton.setGeometry(QtCore.QRect(180, 136, 70, 37))
        self.ConINButton.setObjectName("ConINButton")
        self.LinkInCh = QtWidgets.QCheckBox(self.centralwidget)
        self.LinkInCh.setEnabled(False)
        self.LinkInCh.setGeometry(QtCore.QRect(208, 175, 35, 26))
        self.LinkInCh.setText("")
        self.LinkInCh.setChecked(True)
        self.LinkInCh.setObjectName("LinkInCh")
        self.LinkOutCh = QtWidgets.QCheckBox(self.centralwidget)
        self.LinkOutCh.setEnabled(False)
        self.LinkOutCh.setGeometry(QtCore.QRect(210, 376, 35, 26))
        self.LinkOutCh.setText("")
        self.LinkOutCh.setChecked(True)
        self.LinkOutCh.setObjectName("LinkOutCh")
        self.ProOUTButton = QtWidgets.QPushButton(self.centralwidget)
        self.ProOUTButton.setEnabled(False)
        self.ProOUTButton.setGeometry(QtCore.QRect(180, 297, 70, 37))
        self.ProOUTButton.setObjectName("ProOUTButton")
        self.ConOUTButton = QtWidgets.QPushButton(self.centralwidget)
        self.ConOUTButton.setEnabled(False)
        self.ConOUTButton.setGeometry(QtCore.QRect(180, 337, 70, 37))
        self.ConOUTButton.setObjectName("ConOUTButton")
        self.LucidReadButton = QtWidgets.QPushButton(self.centralwidget)
        self.LucidReadButton.setGeometry(QtCore.QRect(69, 324, 70, 38))
        self.LucidReadButton.setObjectName("LucidReadButton")
        self.LucidWriteButton = QtWidgets.QPushButton(self.centralwidget)
        self.LucidWriteButton.setEnabled(False)
        self.LucidWriteButton.setGeometry(QtCore.QRect(69, 364, 70, 38))
        self.LucidWriteButton.setObjectName("LucidWriteButton")
        self.labelIN_1 = xglucidLabel(self.centralwidget)
        self.labelIN_1.setGeometry(QtCore.QRect(285, 219, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelIN_1.sizePolicy().hasHeightForWidth())
        self.labelIN_1.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelIN_1.setFont(font)
        self.labelIN_1.setText("")
        self.labelIN_1.setObjectName("labelIN_1")
        self.labelIN_2 = xglucidLabel(self.centralwidget)
        self.labelIN_2.setGeometry(QtCore.QRect(340, 219, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelIN_2.sizePolicy().hasHeightForWidth())
        self.labelIN_2.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelIN_2.setFont(font)
        self.labelIN_2.setText("")
        self.labelIN_2.setObjectName("labelIN_2")
        self.labelIN_4 = xglucidLabel(self.centralwidget)
        self.labelIN_4.setGeometry(QtCore.QRect(449, 219, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelIN_4.sizePolicy().hasHeightForWidth())
        self.labelIN_4.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelIN_4.setFont(font)
        self.labelIN_4.setText("")
        self.labelIN_4.setObjectName("labelIN_4")
        self.labelIN_3 = xglucidLabel(self.centralwidget)
        self.labelIN_3.setGeometry(QtCore.QRect(395, 219, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelIN_3.sizePolicy().hasHeightForWidth())
        self.labelIN_3.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelIN_3.setFont(font)
        self.labelIN_3.setText("")
        self.labelIN_3.setObjectName("labelIN_3")
        self.labelIN_6 = xglucidLabel(self.centralwidget)
        self.labelIN_6.setGeometry(QtCore.QRect(560, 219, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelIN_6.sizePolicy().hasHeightForWidth())
        self.labelIN_6.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelIN_6.setFont(font)
        self.labelIN_6.setText("")
        self.labelIN_6.setObjectName("labelIN_6")
        self.labelIN_8 = xglucidLabel(self.centralwidget)
        self.labelIN_8.setGeometry(QtCore.QRect(667, 219, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelIN_8.sizePolicy().hasHeightForWidth())
        self.labelIN_8.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelIN_8.setFont(font)
        self.labelIN_8.setText("")
        self.labelIN_8.setObjectName("labelIN_8")
        self.labelIN_5 = xglucidLabel(self.centralwidget)
        self.labelIN_5.setGeometry(QtCore.QRect(505, 219, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelIN_5.sizePolicy().hasHeightForWidth())
        self.labelIN_5.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelIN_5.setFont(font)
        self.labelIN_5.setText("")
        self.labelIN_5.setObjectName("labelIN_5")
        self.labelIN_7 = xglucidLabel(self.centralwidget)
        self.labelIN_7.setGeometry(QtCore.QRect(612, 219, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelIN_7.sizePolicy().hasHeightForWidth())
        self.labelIN_7.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelIN_7.setFont(font)
        self.labelIN_7.setText("")
        self.labelIN_7.setObjectName("labelIN_7")
        self.labelOUT_2 = xglucidLabel(self.centralwidget)
        self.labelOUT_2.setGeometry(QtCore.QRect(340, 415, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelOUT_2.sizePolicy().hasHeightForWidth())
        self.labelOUT_2.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelOUT_2.setFont(font)
        self.labelOUT_2.setText("")
        self.labelOUT_2.setObjectName("labelOUT_2")
        self.labelOUT_8 = xglucidLabel(self.centralwidget)
        self.labelOUT_8.setGeometry(QtCore.QRect(671, 415, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelOUT_8.sizePolicy().hasHeightForWidth())
        self.labelOUT_8.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelOUT_8.setFont(font)
        self.labelOUT_8.setText("")
        self.labelOUT_8.setObjectName("labelOUT_8")
        self.labelOUT_4 = xglucidLabel(self.centralwidget)
        self.labelOUT_4.setGeometry(QtCore.QRect(451, 415, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelOUT_4.sizePolicy().hasHeightForWidth())
        self.labelOUT_4.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelOUT_4.setFont(font)
        self.labelOUT_4.setText("")
        self.labelOUT_4.setObjectName("labelOUT_4")
        self.labelOUT_1 = xglucidLabel(self.centralwidget)
        self.labelOUT_1.setGeometry(QtCore.QRect(286, 415, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelOUT_1.sizePolicy().hasHeightForWidth())
        self.labelOUT_1.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelOUT_1.setFont(font)
        self.labelOUT_1.setText("")
        self.labelOUT_1.setObjectName("labelOUT_1")
        self.labelOUT_7 = xglucidLabel(self.centralwidget)
        self.labelOUT_7.setGeometry(QtCore.QRect(618, 415, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelOUT_7.sizePolicy().hasHeightForWidth())
        self.labelOUT_7.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelOUT_7.setFont(font)
        self.labelOUT_7.setText("")
        self.labelOUT_7.setObjectName("labelOUT_7")
        self.labelOUT_5 = xglucidLabel(self.centralwidget)
        self.labelOUT_5.setGeometry(QtCore.QRect(506, 415, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelOUT_5.sizePolicy().hasHeightForWidth())
        self.labelOUT_5.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelOUT_5.setFont(font)
        self.labelOUT_5.setText("")
        self.labelOUT_5.setObjectName("labelOUT_5")
        self.labelOUT_6 = xglucidLabel(self.centralwidget)
        self.labelOUT_6.setGeometry(QtCore.QRect(561, 415, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelOUT_6.sizePolicy().hasHeightForWidth())
        self.labelOUT_6.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelOUT_6.setFont(font)
        self.labelOUT_6.setText("")
        self.labelOUT_6.setObjectName("labelOUT_6")
        self.labelOUT_3 = xglucidLabel(self.centralwidget)
        self.labelOUT_3.setGeometry(QtCore.QRect(395, 415, 41, 21))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelOUT_3.sizePolicy().hasHeightForWidth())
        self.labelOUT_3.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.labelOUT_3.setFont(font)
        self.labelOUT_3.setText("")
        self.labelOUT_3.setObjectName("labelOUT_3")
        self.label_18 = QtWidgets.QLabel(self.centralwidget)
        self.label_18.setGeometry(QtCore.QRect(0, -209, 900, 516))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_18.sizePolicy().hasHeightForWidth())
        self.label_18.setSizePolicy(sizePolicy)
        self.label_18.setMaximumSize(QtCore.QSize(904, 895))
        self.label_18.setObjectName("label_18")
        self.SerialPortCombo = QtWidgets.QComboBox(self.centralwidget)
        self.SerialPortCombo.setGeometry(QtCore.QRect(65, 227, 81, 28))
        font = QtGui.QFont()
        font.setPointSize(6)
        self.SerialPortCombo.setFont(font)
        self.SerialPortCombo.setObjectName("SerialPortCombo")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.SerialPortCombo.addItem("")
        self.connection_label = QtWidgets.QLabel(self.centralwidget)
        self.connection_label.setGeometry(QtCore.QRect(72, 270, 71, 20))
        self.connection_label.setObjectName("connection_label")
        self.label_18.raise_()
        self.label.raise_()
        self.labelIN_1.raise_()
        self.labelIN_2.raise_()
        self.labelIN_4.raise_()
        self.labelIN_3.raise_()
        self.labelIN_6.raise_()
        self.labelIN_8.raise_()
        self.labelIN_5.raise_()
        self.labelIN_7.raise_()
        self.labelOUT_2.raise_()
        self.labelOUT_8.raise_()
        self.labelOUT_4.raise_()
        self.labelOUT_1.raise_()
        self.labelOUT_7.raise_()
        self.labelOUT_5.raise_()
        self.labelOUT_6.raise_()
        self.labelOUT_3.raise_()
        self.SliderIN_7.raise_()
        self.SliderIN_3.raise_()
        self.SliderOUT_8.raise_()
        self.SliderOUT_2.raise_()
        self.ConOUTButton.raise_()
        self.SliderOUT_7.raise_()
        self.SliderOUT_5.raise_()
        self.SliderOUT_3.raise_()
        self.SliderOUT_1.raise_()
        self.SliderIN_1.raise_()
        self.ProINButton.raise_()
        self.LucidReadButton.raise_()
        self.ProOUTButton.raise_()
        self.SliderIN_4.raise_()
        self.LucidWriteButton.raise_()
        self.SliderIN_6.raise_()
        self.ConINButton.raise_()
        self.SliderOUT_6.raise_()
        self.SliderIN_5.raise_()
        self.SliderIN_2.raise_()
        self.SliderIN_8.raise_()
        self.SliderOUT_4.raise_()
        self.LucidOpticalCombo.raise_()
        self.LucidSpdifCombo.raise_()
        self.LinkOutCh.raise_()
        self.LucidIdCombo.raise_()
        self.LucidMeterCombo.raise_()
        self.LucidAesCombo.raise_()
        self.LinkInCh.raise_()
        self.LucidSyncCombo.raise_()
        self.LucidAnalogSrcCombo.raise_()
        self.SerialPortCombo.raise_()
        self.connection_label.raise_()
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.statusbar.sizePolicy().hasHeightForWidth())
        self.statusbar.setSizePolicy(sizePolicy)
        self.statusbar.setSizeGripEnabled(False)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.action_dev_ttyUSB0 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyUSB0.setCheckable(True)
        self.action_dev_ttyUSB0.setChecked(True)
        self.action_dev_ttyUSB0.setObjectName("action_dev_ttyUSB0")
        self.action_dev_ttyUSB1 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyUSB1.setCheckable(True)
        self.action_dev_ttyUSB1.setObjectName("action_dev_ttyUSB1")
        self.action_dev_ttyUSB2 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyUSB2.setCheckable(True)
        self.action_dev_ttyUSB2.setObjectName("action_dev_ttyUSB2")
        self.action_dev_ttyUSB3 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyUSB3.setCheckable(True)
        self.action_dev_ttyUSB3.setObjectName("action_dev_ttyUSB3")
        self.action_dev_ttyUSB4 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyUSB4.setCheckable(True)
        self.action_dev_ttyUSB4.setObjectName("action_dev_ttyUSB4")
        self.action_dev_ttyUSB4_2 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyUSB4_2.setObjectName("action_dev_ttyUSB4_2")
        self.action_dev_ttyUSB5 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyUSB5.setCheckable(True)
        self.action_dev_ttyUSB5.setObjectName("action_dev_ttyUSB5")
        self.action_dev_ttyS0 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyS0.setCheckable(True)
        self.action_dev_ttyS0.setObjectName("action_dev_ttyS0")
        self.action_dev_ttyS1 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyS1.setCheckable(True)
        self.action_dev_ttyS1.setObjectName("action_dev_ttyS1")
        self.action_dev_ttyS2 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyS2.setCheckable(True)
        self.action_dev_ttyS2.setObjectName("action_dev_ttyS2")
        self.action_dev_ttyS3 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyS3.setCheckable(True)
        self.action_dev_ttyS3.setObjectName("action_dev_ttyS3")
        self.action_dev_ttyS4 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyS4.setCheckable(True)
        self.action_dev_ttyS4.setObjectName("action_dev_ttyS4")
        self.action_dev_ttyS5 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyS5.setCheckable(True)
        self.action_dev_ttyS5.setObjectName("action_dev_ttyS5")
        self.action_dev_ttyAMA0 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyAMA0.setCheckable(True)
        self.action_dev_ttyAMA0.setObjectName("action_dev_ttyAMA0")
        self.action_dev_ttyAMA1 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyAMA1.setCheckable(True)
        self.action_dev_ttyAMA1.setObjectName("action_dev_ttyAMA1")
        self.action_dev_ttyACM0 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyACM0.setCheckable(True)
        self.action_dev_ttyACM0.setObjectName("action_dev_ttyACM0")
        self.action_dev_ttyACM1 = QtWidgets.QAction(MainWindow)
        self.action_dev_ttyACM1.setCheckable(True)
        self.action_dev_ttyACM1.setObjectName("action_dev_ttyACM1")
        self.action_dev_rfcomm0 = QtWidgets.QAction(MainWindow)
        self.action_dev_rfcomm0.setCheckable(True)
        self.action_dev_rfcomm0.setObjectName("action_dev_rfcomm0")
        self.action_dev_ircomm0 = QtWidgets.QAction(MainWindow)
        self.action_dev_ircomm0.setCheckable(True)
        self.action_dev_ircomm0.setObjectName("action_dev_ircomm0")
        self.action_dev_cuau0 = QtWidgets.QAction(MainWindow)
        self.action_dev_cuau0.setCheckable(True)
        self.action_dev_cuau0.setObjectName("action_dev_cuau0")

        self.retranslateUi(MainWindow)
        self.SliderIN_1.valueChanged['int'].connect(self.labelIN_1.on_slider_value_changed)
        self.SliderIN_2.valueChanged['int'].connect(self.labelIN_2.on_slider_value_changed)
        self.LucidWriteButton.clicked.connect(self.statusbar.clearMessage)
        self.SliderOUT_1.valueChanged['int'].connect(self.labelOUT_1.on_slider_value_changed)
        self.SliderIN_3.valueChanged['int'].connect(self.labelIN_3.on_slider_value_changed)
        self.SliderIN_4.valueChanged['int'].connect(self.labelIN_4.on_slider_value_changed)
        self.SliderIN_5.valueChanged['int'].connect(self.labelIN_5.on_slider_value_changed)
        self.SliderIN_6.valueChanged['int'].connect(self.labelIN_6.on_slider_value_changed)
        self.SliderIN_7.valueChanged['int'].connect(self.labelIN_7.on_slider_value_changed)
        self.SliderIN_8.valueChanged['int'].connect(self.labelIN_8.on_slider_value_changed)
        self.LucidReadButton.clicked.connect(self.statusbar.clearMessage)
        self.SliderOUT_2.valueChanged['int'].connect(self.labelOUT_2.on_slider_value_changed)
        self.SliderOUT_3.valueChanged['int'].connect(self.labelOUT_3.on_slider_value_changed)
        self.SliderOUT_4.valueChanged['int'].connect(self.labelOUT_4.on_slider_value_changed)
        self.SliderOUT_5.valueChanged['int'].connect(self.labelOUT_5.on_slider_value_changed)
        self.SliderOUT_6.valueChanged['int'].connect(self.labelOUT_6.on_slider_value_changed)
        self.SliderOUT_7.valueChanged['int'].connect(self.labelOUT_7.on_slider_value_changed)
        self.SliderOUT_8.valueChanged['int'].connect(self.labelOUT_8.on_slider_value_changed)
        self.SliderIN_1.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderIN_2.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderIN_3.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderIN_4.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderIN_8.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderIN_7.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderIN_6.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderIN_5.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderOUT_1.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderOUT_8.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderOUT_7.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderOUT_6.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderOUT_5.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderOUT_4.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderOUT_3.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.SliderOUT_2.valueChanged['int'].connect(self.centralwidget.on_glucid_slider_changed)
        self.LucidReadButton.clicked.connect(self.centralwidget.read_button_clicked)
        self.LucidWriteButton.clicked.connect(self.centralwidget.write_button_clicked)
        self.SerialPortCombo.currentTextChanged['QString'].connect(self.centralwidget.on_serial_port_changed)
        self.ProOUTButton.clicked.connect(self.centralwidget.pro_or_consumer_clicked)
        self.ConINButton.clicked.connect(self.centralwidget.pro_or_consumer_clicked)
        self.ConOUTButton.clicked.connect(self.centralwidget.pro_or_consumer_clicked)
        self.ProINButton.clicked.connect(self.centralwidget.pro_or_consumer_clicked)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "xglucid"))
        self.LucidMeterCombo.setItemText(0, _translate("MainWindow", "Analog IN"))
        self.LucidMeterCombo.setItemText(1, _translate("MainWindow", "Digital IN"))
        self.LucidMeterCombo.setItemText(2, _translate("MainWindow", "Analog OUT"))
        self.LucidMeterCombo.setItemText(3, _translate("MainWindow", "Digital OUT"))
        self.LucidSyncCombo.setItemText(0, _translate("MainWindow", "ADAT"))
        self.LucidSyncCombo.setItemText(1, _translate("MainWindow", "WordClock"))
        self.LucidSyncCombo.setItemText(2, _translate("MainWindow", "44.1 Int"))
        self.LucidSyncCombo.setItemText(3, _translate("MainWindow", "48 Int"))
        self.LucidSyncCombo.setItemText(4, _translate("MainWindow", "AES In1"))
        self.LucidSyncCombo.setItemText(5, _translate("MainWindow", "AES In2"))
        self.LucidSyncCombo.setItemText(6, _translate("MainWindow", "AES In3"))
        self.LucidSyncCombo.setItemText(7, _translate("MainWindow", "S/PDIF "))
        self.LucidIdCombo.setItemText(0, _translate("MainWindow", "00"))
        self.LucidIdCombo.setItemText(1, _translate("MainWindow", "01"))
        self.LucidIdCombo.setItemText(2, _translate("MainWindow", "02"))
        self.LucidIdCombo.setItemText(3, _translate("MainWindow", "03"))
        self.LucidIdCombo.setItemText(4, _translate("MainWindow", "04"))
        self.LucidIdCombo.setItemText(5, _translate("MainWindow", "05"))
        self.LucidIdCombo.setItemText(6, _translate("MainWindow", "06"))
        self.LucidIdCombo.setItemText(7, _translate("MainWindow", "07"))
        self.LucidSpdifCombo.setItemText(0, _translate("MainWindow", "1,2 AES"))
        self.LucidSpdifCombo.setItemText(1, _translate("MainWindow", "1,2 S/PDIF"))
        self.LucidAnalogSrcCombo.setItemText(0, _translate("MainWindow", "ADAT IN"))
        self.LucidAnalogSrcCombo.setItemText(1, _translate("MainWindow", "AES IN"))
        self.LucidOpticalCombo.setItemText(0, _translate("MainWindow", "Analog IN"))
        self.LucidOpticalCombo.setItemText(1, _translate("MainWindow", "AES IN"))
        self.LucidAesCombo.setItemText(0, _translate("MainWindow", "ADAT IN"))
        self.LucidAesCombo.setItemText(1, _translate("MainWindow", "Analog IN"))
        self.ProINButton.setText(_translate("MainWindow", "+4 dBu"))
        self.ConINButton.setText(_translate("MainWindow", "-10 dBV"))
        self.ProOUTButton.setText(_translate("MainWindow", "+4 dBu"))
        self.ConOUTButton.setText(_translate("MainWindow", "-10 dBV"))
        self.LucidReadButton.setText(_translate("MainWindow", "READ"))
        self.LucidWriteButton.setText(_translate("MainWindow", "WRITE"))
        self.label_18.setText(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Cantarell\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" dir=\'rtl\' style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><img src=\":/newPrefix/glucidLayout2-smallsig.png\" /></p></body></html>"))
        self.SerialPortCombo.setItemText(0, _translate("MainWindow", "/dev/ttyUSB0"))
        self.SerialPortCombo.setItemText(1, _translate("MainWindow", "/dev/tty.usbserial"))
        self.SerialPortCombo.setItemText(2, _translate("MainWindow", "/dev/cu.usbserial"))
        self.SerialPortCombo.setItemText(3, _translate("MainWindow", "COM1"))
        self.SerialPortCombo.setItemText(4, _translate("MainWindow", "/dev/midi1"))
        self.SerialPortCombo.setItemText(5, _translate("MainWindow", "/dev/snd/midiC1D0"))
        self.SerialPortCombo.setItemText(6, _translate("MainWindow", "MIDIOUT2"))
        self.SerialPortCombo.setItemText(7, _translate("MainWindow", "/dev/ttyUSB1"))
        self.SerialPortCombo.setItemText(8, _translate("MainWindow", "/dev/ttyUSB2"))
        self.SerialPortCombo.setItemText(9, _translate("MainWindow", "/dev/ttyUSB3"))
        self.SerialPortCombo.setItemText(10, _translate("MainWindow", "/dev/ttyUSB4"))
        self.SerialPortCombo.setItemText(11, _translate("MainWindow", "/dev/ttyUSB5"))
        self.SerialPortCombo.setItemText(12, _translate("MainWindow", "/dev/ttyS0"))
        self.SerialPortCombo.setItemText(13, _translate("MainWindow", "/dev/ttyS1"))
        self.SerialPortCombo.setItemText(14, _translate("MainWindow", "/dev/ttyS2"))
        self.SerialPortCombo.setItemText(15, _translate("MainWindow", "/dev/ttyS3"))
        self.SerialPortCombo.setItemText(16, _translate("MainWindow", "/dev/ttyS4"))
        self.SerialPortCombo.setItemText(17, _translate("MainWindow", "/dev/ttyS5"))
        self.SerialPortCombo.setItemText(18, _translate("MainWindow", "/dev/ttyAMA0"))
        self.SerialPortCombo.setItemText(19, _translate("MainWindow", "/dev/ttyAMA1"))
        self.SerialPortCombo.setItemText(20, _translate("MainWindow", "/dev/ttyACM0"))
        self.SerialPortCombo.setItemText(21, _translate("MainWindow", "/dev/ttyACM1"))
        self.SerialPortCombo.setItemText(22, _translate("MainWindow", "/dev/rfcomm0"))
        self.SerialPortCombo.setItemText(23, _translate("MainWindow", "/dev/rfcomm1"))
        self.SerialPortCombo.setItemText(24, _translate("MainWindow", "/dev/ircomm0"))
        self.SerialPortCombo.setItemText(25, _translate("MainWindow", "/dev/ircomm1"))
        self.SerialPortCombo.setItemText(26, _translate("MainWindow", "/dev/cuau0"))
        self.SerialPortCombo.setItemText(27, _translate("MainWindow", "/dev/cuau1"))
        self.SerialPortCombo.setItemText(28, _translate("MainWindow", "/dev/cuau2"))
        self.SerialPortCombo.setItemText(29, _translate("MainWindow", "/dev/cuaU0"))
        self.SerialPortCombo.setItemText(30, _translate("MainWindow", "/dev/cuaU1"))
        self.SerialPortCombo.setItemText(31, _translate("MainWindow", "COM1"))
        self.SerialPortCombo.setItemText(32, _translate("MainWindow", "COM2"))
        self.SerialPortCombo.setItemText(33, _translate("MainWindow", "COM3"))
        self.SerialPortCombo.setItemText(34, _translate("MainWindow", "COM4"))
        self.connection_label.setText(_translate("MainWindow", "NONE"))
        self.action_dev_ttyUSB0.setText(_translate("MainWindow", "/dev/ttyUSB0"))
        self.action_dev_ttyUSB1.setText(_translate("MainWindow", "/dev/ttyUSB1"))
        self.action_dev_ttyUSB2.setText(_translate("MainWindow", "/dev/ttyUSB2"))
        self.action_dev_ttyUSB3.setText(_translate("MainWindow", "/dev/ttyUSB3"))
        self.action_dev_ttyUSB4.setText(_translate("MainWindow", "/dev/ttyUSB4"))
        self.action_dev_ttyUSB4_2.setText(_translate("MainWindow", "/dev/ttyUSB4"))
        self.action_dev_ttyUSB5.setText(_translate("MainWindow", "/dev/ttyUSB5"))
        self.action_dev_ttyS0.setText(_translate("MainWindow", "/dev/ttyS0"))
        self.action_dev_ttyS1.setText(_translate("MainWindow", "/dev/ttyS1"))
        self.action_dev_ttyS2.setText(_translate("MainWindow", "/dev/ttyS2"))
        self.action_dev_ttyS3.setText(_translate("MainWindow", "/dev/ttyS3"))
        self.action_dev_ttyS4.setText(_translate("MainWindow", "/dev/ttyS4"))
        self.action_dev_ttyS5.setText(_translate("MainWindow", "/dev/ttyS5"))
        self.action_dev_ttyAMA0.setText(_translate("MainWindow", "/dev/ttyAMA0"))
        self.action_dev_ttyAMA1.setText(_translate("MainWindow", "/dev/ttyAMA1"))
        self.action_dev_ttyACM0.setText(_translate("MainWindow", "/dev/ttyACM0"))
        self.action_dev_ttyACM1.setText(_translate("MainWindow", "/dev/ttyACM1"))
        self.action_dev_rfcomm0.setText(_translate("MainWindow", "/dev/rfcomm0"))
        self.action_dev_ircomm0.setText(_translate("MainWindow", "/dev/ircomm0"))
        self.action_dev_cuau0.setText(_translate("MainWindow", "/dev/cuau0"))

from glucid.xglucidUIWidgets import xglucidLabel, xglucidWidget
from glucid import Glucid8824_rc



"""xglucidUIWidgets provides custom classes for QtWidget
behavoir in glucid

xglucid is a part of the glucid package and provides
a qt Grpahical interface to configure a Lucid 8824
Analog/Digital Audio Converter RS232 interface over
a serial port.

Copyright (C) 2017,2018  Daniel R Mechanic (dan.mechanic@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License ONLY.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""
from glucid import xglucidUIWidgets
from glucid.glucid8824 import Glucid8824
from PyQt5.QtCore import QCoreApplication
from PyQt5.QtGui import QPainter, QColor, QFont
from PyQt5.QtWidgets import (QWidget, QLabel, QSlider, QComboBox,
                             QCheckBox, QPushButton)

class xglucidWidget(QWidget):
    """xglucidWidget extends QWidget to provide custom signals
    and slots fo the glucid interface
    """
    def __init__(self, parent=None):
        super().__init__(parent)
        QWidget.__init__(self, parent)

        self.InputSliders = []
        self.OutputSliders = []

        self.myLucid = Glucid8824()
        #self.myLucid.set_device_from_configfile()
        self.initUI()
        
    def initUI(self):
        """set geometry and title, call show()"""
        self.setGeometry(0, 0, 897, 471)
        self.setWindowTitle('xglucid')
        self.show()
        # update device selector
        


    def need_write_status(self):
        """Deprecated: `Needs write` written to status bar"""
        self.parent().statusBar().showMessage('Need to Write to the Device')

    def make_sliders(self):
        """Build lists `InputSliders` and `OutputSliders`.  This is
        a convenience function so these sliders can be linked together
        """
        self.InputSliders = [
            self.parent().SliderIN_1,
            self.parent().SliderIN_2,
            self.parent().SliderIN_3,
            self.parent().SliderIN_4,
            self.parent().SliderIN_5,
            self.parent().SliderIN_6,
            self.parent().SliderIN_7,
            self.parent().SliderIN_8,
        ]
        self.OutputSliders = [
            self.parent().SliderOUT_1,
            self.parent().SliderOUT_2,
            self.parent().SliderOUT_3,
            self.parent().SliderOUT_4,
            self.parent().SliderOUT_5,
            self.parent().SliderOUT_6,
            self.parent().SliderOUT_7,
            self.parent().SliderOUT_8,
        ]

    def on_glucid_slider_changed(self, value):
        """when a slider is changed, we check
        if the linked box is checked, and if so
        we set the values of other sliders...

        this may be better in a slider class
        as a slot routine...
        """
        
        changed_slider = self.sender()
        self.make_sliders()

        if (changed_slider in self.InputSliders and
            self.parent().LinkInCh.isChecked()):
            for sl in self.InputSliders:
                if sl is not changed_slider:
                    sl.setValue(value)
        if (changed_slider in self.OutputSliders and
        self.parent().LinkOutCh.isChecked()):
            for sl in self.OutputSliders:
                if sl is not changed_slider:
                    sl.setValue(value)

    def on_serial_port_changed(self, value):
        """Declare a new Glucid8824 object with the 
        new serial interface
        """
        # TODO: implement set_iface in glucid?
        self.myLucid = Glucid8824(siface=value)
        self.disable_all_except_comm()

    def on_device_id_changed(self, value):
        """Update device_id """
        self.myLucid.LucidID = value
        self.disable_all_except_comm()
        
    def disable_all_except_comm(self):
        DontDisable = [
            "SerialPortCombo",
            "LucidIdCombo",
            "LucidReadButton",
            "labelIN_1",
            "labelIN_2",
            "labelIN_3",
            "labelIN_4",
            "labelIN_5",
            "labelIN_6",
            "labelIN_7",
            "labelIN_8",
            "labelOUT_1",
            "labelOUT_2",
            "labelOUT_3",
            "labelOUT_4",
            "labelOUT_5",
            "labelOUT_6",
            "labelOUT_7",
            "labelOUT_8",
        ]
        for i in self.parent().centralwidget.children():
                if i.objectName() not in DontDisable:
                    i.setEnabled(False)

    def enable_all_children(self):
        """Deprecated: delete this"""
        for i in self.parent().centralwidget.children():
            i.setEnabled(True)

    def read_button_clicked(self):
        """The read button was clicked, if successful set all
        widgets to proper values, otherwise reflect the failure
        in the status bar
        """
        try:
            # we only connect to check connectivity, then close
            if self.myLucid.connect():
                self.parent().statusBar().showMessage("Connected using %s" %
                                                      self.myLucid.get_iface())
                self.myLucid.disconnect()
                self.set_ui_from_lucid()
            else:
                self.parent().statusBar().showMessage(
                    "FAILED to connect using %s" %
                    self.myLucid.get_iface())
        except:
            self.parent().statusBar().showMessage("Connection FAILED: %s" %
                                                      self.myLucid.get_iface())
            self.disable_all_except_comm()
 
            
    def set_ui_from_lucid(self):
        """Given we are connected to a lucid, set all UI values 
        by reading each individually from the lucid, setting the
        widget value, and enabling the widget
        """

        self.disable_all_except_comm()
        # make sure disconnecting first
        self.myLucid.disconnect()
        if self.myLucid.connect():
            self.parent().statusBar().showMessage(
                "Connected using %s and reading DATA..." %
                self.myLucid.get_iface())
        else:
            return -1

        QCoreApplication.processEvents()
        # Set Clock Sync
        self.parent().statusBar().showMessage(
            "Connected using %s and reading Clock Sync..." %
            self.myLucid.get_iface())

        QCoreApplication.processEvents()
        self.parent().centralwidget.findChild(
            QComboBox, "LucidSyncCombo"
        ).setCurrentIndex(self.myLucid.get_sync_source(False))
        self.parent().centralwidget.findChild(
            QComboBox, "LucidSyncCombo"
        ).setEnabled(True)

        # set device_id
        self.parent().centralwidget.findChild(
            QComboBox, "LucidIdCombo"
        ).setCurrentIndex(int(self.myLucid.get_instanceid()))
        self.parent().centralwidget.findChild(
            QComboBox, "LucidIdCombo"
        ).setEnabled(True)

        # set Metering
        self.parent().statusBar().showMessage(
            "Connected using %s and reading Front Meter..." %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        self.parent().centralwidget.findChild(
            QComboBox, "LucidMeterCombo"
        ).setCurrentIndex(self.myLucid.get_meter(False))
        self.parent().centralwidget.findChild(
            QComboBox, "LucidMeterCombo"
        ).setEnabled(True)
        QCoreApplication.processEvents()
        # set optical out
        self.parent().statusBar().showMessage(
            "Connected using %s and reading ADAT OUT source..." %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        self.parent().centralwidget.findChild(
            QComboBox, "LucidOpticalCombo"
        ).setCurrentIndex(self.myLucid.get_opt_source(False))
        self.parent().centralwidget.findChild(
            QComboBox, "LucidOpticalCombo"
        ).setEnabled(True)

        # set AES out
        # NOTE: SETTING AES DOES NOT APPEAR TO WORK
        self.parent().statusBar().showMessage(
            "%s : reading AES OUT source... WARNING: READ-ONLY Value" %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        self.parent().centralwidget.findChild(
            QComboBox, "LucidOpticalCombo"
        ).setCurrentIndex(self.myLucid.get_opt_source(False))
        # Leave disabled because you cannot set this BUG
        # self.parent().centralwidget.findChild(QComboBox,
        #                            "LucidOpticalCombo").setEnabled(True)

        # set anlog out src
        self.parent().statusBar().showMessage(
            "Connected using %s and reading Analog OUT source..." %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        self.parent().centralwidget.findChild(
            QComboBox, "LucidAnalogSrcCombo"
        ).setCurrentIndex(self.myLucid.get_analog_source(False))
        self.parent().centralwidget.findChild(
            QComboBox, "LucidAnalogSrcCombo"
        ).setEnabled(True)

        # set digital in 1,2
        self.parent().statusBar().showMessage(
            "Connected using %s and reading Digital Ch 1,2 source..." %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        self.parent().centralwidget.findChild(
            QComboBox, "LucidSpdifCombo"
        ).setCurrentIndex(self.myLucid.get_dig1(False))
        self.parent().centralwidget.findChild(
            QComboBox, "LucidSpdifCombo"
        ).setEnabled(True)

        # set deviceId
        self.parent().statusBar().showMessage(
            "Connected using %s and reading DEVICE ID..." %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        self.parent().centralwidget.findChild(
            QComboBox, "LucidIdCombo"
        ).setCurrentIndex(int(self.myLucid.get_instanceid()))

        self.parent().centralwidget.findChild(
            QComboBox, "LucidIdCombo").setEnabled(True)

        # set input and output sliders
        self.parent().statusBar().showMessage(
            "Connected using %s and reading Channel Levels..." %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        
        # turn on other buttons...
        self.parent().centralwidget.findChild(
            QCheckBox, "LinkInCh").setEnabled(True)
        self.parent().centralwidget.findChild(
            QCheckBox, "LinkOutCh").setEnabled(True)
        self.parent().centralwidget.findChild(
            QCheckBox, "LinkInCh").setCheckState(self.myLucid.is_in_linked())
        self.parent().centralwidget.findChild(
            QCheckBox, "LinkOutCh").setCheckState(self.myLucid.is_out_linked())

        self.myLucid.get_gain()

        # we can disconnect here:
        self.myLucid.disconnect()
        
        self.make_sliders()
        
        for i in range(0, 8):
            instr = "SliderIN_"+str(i+1)
            outstr = "SliderOUT_"+str(i+1)

            self.parent().centralwidget.findChild(
                QSlider, instr).setEnabled(True)
            self.parent().centralwidget.findChild(
                QSlider, outstr).setEnabled(True)

            # hack to get all labels to redraw no matter previous val
            # wont work if previous value was -95... ok... that's silent
            self.parent().centralwidget.findChild(QSlider, instr).setValue(
                -95)
            self.parent().centralwidget.findChild(QSlider, outstr).setValue(
                -95)
            # set to gainlist values
            self.parent().centralwidget.findChild(QSlider, instr).setValue(
                int(self.myLucid.gainlist[i]-96))
            self.parent().centralwidget.findChild(QSlider, outstr).setValue(
                int(self.myLucid.gainlist[i+8]-96))



        self.parent().centralwidget.findChild(
            QPushButton, "ProINButton").setEnabled(True)
        self.parent().centralwidget.findChild(
            QPushButton, "ProOUTButton").setEnabled(True)
        self.parent().centralwidget.findChild(
            QPushButton, "ConINButton").setEnabled(True)
        self.parent().centralwidget.findChild(
            QPushButton, "ConOUTButton").setEnabled(True)
        self.parent().centralwidget.findChild(
            QPushButton, "LucidWriteButton").setEnabled(True)
        self.parent().statusBar().showMessage("Finished Reading DATA")
        QCoreApplication.processEvents()
        # make sure to disconnect


        
    def pro_or_consumer_clicked(self):
        """a slot to handle when a user clicks the +4 or -10 buttons"""
        # There's a bug here
        # if the first Slider is already the correct
        # value, no update is triggered.
        # This should be reimplemented in the slider class
        # so that sliders themselves have a slot that listens
        # to the buttons signal.
        if self.sender().objectName() == "ProOUTButton":
            self.parent().centralwidget.findChild(
                QCheckBox, "LinkOutCh").setCheckState(True)
            self.parent().centralwidget.findChild(
                QSlider, "SliderOUT_1").setValue(1)
        elif self.sender().objectName() == "ConOUTButton":
            self.parent().centralwidget.findChild(
                QCheckBox, "LinkOutCh").setCheckState(True)
            self.parent().centralwidget.findChild(
                QSlider, "SliderOUT_1").setValue(-11)
        elif self.sender().objectName() == "ProINButton":
            self.parent().centralwidget.findChild(
                QCheckBox, "LinkInCh").setCheckState(True)
            self.parent().centralwidget.findChild(
                QSlider, "SliderIN_1").setValue(-8)
        elif self.sender().objectName() == "ConINButton":
            self.parent().centralwidget.findChild(
                QCheckBox, "LinkInCh").setCheckState(True)
            self.parent().centralwidget.findChild(
                QSlider, "SliderIN_1").setValue(4)

    def write_ui_to_lucid(self):
        """Given we are connected to a lucid, write all UI values to it
        
        We write all values, and not just updated ones because
        something may have changed on the unit since we last read them
        and writing what the user sees would be the most predictable 
        behavoir

        however, it's slow.
        """
        self.disable_all_except_comm()
        # make sure diconnected first...
        self.myLucid.disconnect()

        if self.myLucid.connect():
            self.parent().statusBar().showMessage(
                "Connected using %s id: %s and writing DATA..." %
                (self.myLucid.get_iface(),
                 self.myLucid.get_instanceid()) )
            QCoreApplication.processEvents()
            self.myLucid.glucidconf['DEFAULT']['Device']=self.myLucid.get_iface()
            self.myLucid.glucidconf['DEFAULT']['DEVICE_ID']=self.myLucid.get_instanceid()
        else:
            self.parent().statusBar().showMessage(
                "ERROR: Unable to connect to %s id %s to write DATA" %
                (self.myLucid.get_iface(),
                 self.myLucid.get_instancid()))            
            return -1
        
        
        # Set Clock Sync
        self.parent().statusBar().showMessage(
            "Connected using %s and writing Clock Sync..." %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        self.myLucid.set_sync_source(
            self.parent().centralwidget.findChild(
                QComboBox, "LucidSyncCombo").currentIndex())



        # # set Metering
        # self.parent().statusBar().showMessage(
        #     "Connected using %s and writing Front Meter..." %
        #     self.myLucid.get_iface())
        # self.myLucid.set_meter(
        #     self.parent().centralwidget.findChild(
        #         QComboBox, "LucidMeterCombo").currentIndex())

        # # set digital in 1,2
        # self.parent().statusBar().showMessage(
        #     "Connected using %s and writing Digital Ch 1,2 source..." %
        #     self.myLucid.get_iface())

        # self.myLucid.set_dig1(
        #     self.parent().centralwidget.findChild(
        #         QComboBox, "LucidSpdifCombo").currentIndex())

        ### Updated to use set_meter_and_dig1
        self.parent().statusBar().showMessage(
            "Connected using %s and writing mode..." %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        self.myLucid.set_meter_and_dig1(
            (4 * self.parent().centralwidget.findChild(
                QComboBox, "LucidSpdifCombo").currentIndex()) +
            self.parent().centralwidget.findChild(
                QComboBox, "LucidMeterCombo").currentIndex())
        
        # set optical out
        self.parent().statusBar().showMessage(
            "Connected using %s and writing ADAT OUT source..." %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        self.myLucid.set_opt_src(
            self.parent().centralwidget.findChild(
                QComboBox, "LucidOpticalCombo").currentIndex())

        # set AES out
        # NOTE: SETTING AES DOES NOT APPEAR TO WORK
        #

        # set anlog out src
        self.parent().statusBar().showMessage(
            "Connected using %s and writing Analog OUT source..." %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        self.myLucid.set_analog_src(
            self.parent().centralwidget.findChild(
                QComboBox, "LucidAnalogSrcCombo").currentIndex())


        # save state of sliders, if they are linked
        if self.parent().centralwidget.findChild(QCheckBox, "LinkInCh").isChecked():
            self.myLucid.glucidconf['DEFAULT']['LINKINCH']='1'
        else:
            self.myLucid.glucidconf['DEFAULT']['LINKINCH']='0'

        if self.parent().centralwidget.findChild(QCheckBox, "LinkOutCh").isChecked():
            self.myLucid.glucidconf['DEFAULT']['LINKOUTCH']='1'
        else:
            self.myLucid.glucidconf['DEFAULT']['LINKOUTCH']='0'
            
            
        # set input and output sliders
        self.parent().statusBar().showMessage(
            "Connected using %s and writing Channel Levels..." %
            self.myLucid.get_iface())
        QCoreApplication.processEvents()
        for i in range(1, 9):
            self.myLucid.update_channel_in_gainlist(
                self.parent().centralwidget.findChild(
                    QSlider, "SliderIN_"+str(i)).value(), i-1
            )
            self.myLucid.update_channel_in_gainlist(
                self.parent().centralwidget.findChild(
                    QSlider, "SliderOUT_"+str(i)).value(), i+7
            )

        self.myLucid.write_gainlist_to_lucid()
        
        self.parent().statusBar().showMessage("Finished Writing DATA")
        QCoreApplication.processEvents()
        # windows fails without disconnecting first
        self.myLucid.disconnect()
        #self.myLucid.write_configfile()

        # instead of setting the ui from the lucid,
        # which takes awhile...  let's leave the UI
        # alone but disabled, causing the user to click
        # 'read' again.
        self.disable_all_except_comm()
        #self.set_ui_from_lucid()
        return 1

    def write_button_clicked(self):
        """The write button was clicked, if successful set all
        widgets to proper values, otherwise reflect the failure
        in the status bar
        """
        self.disable_all_except_comm()

        if self.myLucid.connect():
            self.parent().statusBar().showMessage("Connected using %s" %
                                                  self.myLucid.get_iface())
            if self.write_ui_to_lucid() < 0:
                self.parent().statusBar().showMessage("ERROR: Failed to write")
        else:
            self.parent().statusBar().showMessage("FAILED to connect using %s" %
                                                  self.myLucid.get_iface())


class xglucidLabel(QLabel):
    """A class to be used to label qglucid sliders"""
    
    def __init__(self, parent=None):
        super().__init__(parent)

    def on_slider_value_changed(self, value):
        """setText with the value of `value` with
        positive values preceded with a `+` and
        ended with `dB`
        """

        if self.sender().value() > 0:
            retstring = str('+'+str(self.sender().value())+"dB")
            self.setText(retstring)
        else:
            self.setText(str(self.sender().value())+"dB")
        QCoreApplication.processEvents()


class xglucidInputSlider(QSlider):
    """Not used;  something like this should be used for the
    linked channel behavoir
    """
    def __init__(self, parent=None):
        super().__init__(parent)

    def on_input_slider_value_changed(self, value):

        if self.sender().value() > 0:
            retstring = str('+'+str(self.sender().value())+"dB")
            self.setText(retstring)
        else:
            self.setText(str(self.sender().value())+"dB")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
glucid8824.py defines a class `glucid8824` which
implements an api to a Lucid 8824 Analog/Digital Audio Converter
RS232 interface over a serial port.

A `main` and associated functions provide a useful
command line tool `glucid` in glucid.py to configure the Lucid 8824/.

Copyright (C) 2017,2018  Daniel R Mechanic (dan.mechanic@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License ONLY.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

__version__ = '0.4.0a'
__author__ = 'Daniel R Mechanic (dan.mechanic@gmail.com)'
CONFIGFILE='.glucid.cfg'

import configparser
import getopt
import logging
import os
import sys
from glucid.glucid8824 import Glucid8824

# BEGIN FUNCTIONS FOR COMMAND LINE INTERFACE

def banner():
    progname = str(sys.argv[0].split('/')[-1])
    print('-'*40)
    # print("glucid: The Lucid 8824 Tool")
    # print("%s version %s" %
    #      (progname, str(__version__))
    #      )
    print("\nglucid and glucid8824.py:")
    print("Copyright (C) 2017,2018 Daniel R Mechanic")
    print("GPL version 3 ONLY <http://gnu.org/licenses/gpl.html>.")
    print("This program comes with ABSOLUTELY NO WARRANTY;")
    print("This is free software, and you are welcome to")
    print("change and redistribute it under certain circumstances;")
    print("For details see LICENSE")
    print('-'*40)


def usage_light():
    print("Usage: "+sys.argv[0].split('/')[-1]+" [OPTION]... ")
    print("Try '"+sys.argv[0].split('/')[-1]+" --help' for more information")


def usage():
    print("")
    print("Usage: "+sys.argv[0].split('/')[-1]+" [OPTION]... ")
    print("Get or Set Values on a Lucid8824 Interface")
    print("")
    print("OPTIONS:")
    print("  -h,--help\tthis help")
    print("  -v,--verbose\tverbose output")
    print("  -d DEVICE\tUse DEVICE instead of /dev/ttyUSB0")
    print("  -D DEVICE\tUse DEVICE instead of /dev/ttyUSB0")
    print("         \tAND Set DEVICE as new default")
    print("\nSET OPTIONS:")
    print("  --set_sync=SYNC\tset source to sync to SYNC")
    print("        \tSYNC is a number 0-7:")
    print("         \t0 -'ADAT'")
    print("         \t1 -'WordClock'")
    print("         \t2 -'44.1 internal'")
    print("         \t3 -'48 Internal'")
    print("         \t4 -'AES In1'")
    print("         \t5 -'AES In2'")
    print("         \t6 -'AES In3'")
    print("         \t7 -'S/PDIF In'")
    print("")
    print("  --set_analog=SRC\tset analog output source to SRC")
    print("        \tSRC is a number 0 or 1:")
    print("         \t0 -'ADAT In'")
    print("         \t1 -'AES In'")
    print("")
    # TODO: This is commented out because it does not appear to work :-/
    # print("  --set_aes=SRC\tset aes output source to SRC")
    # print("        \tSRC is a number 0 or 1:")
    # print("         \t0 -'ADAT In'")
    # print("         \t1 -'Analog In'")
    # print("")
    print("  --set_opt=SRC\tset optical output source to SRC")
    print("        \tSRC is a number 0 or 1:")
    print("         \t0 -'Analog In'")
    print("         \t1 -'AES In'")
    print("")
    print("  --set_meter_and_dig1=VALUE\tset meters and dig 1/2")
    print("        \tVALUE is a number 0-7:")
    print("         \t0 -'Meter Analog In, use aes for 1,2'")
    print("         \t1 -'Meter Digital In, use aes for 1,2'")
    print("         \t2 -'Analog Out, use aes for 1,2'")
    print("         \t3 -'Digital Out, use aes for 1,2'")
    print("         \t4 -'Meter Analog In, use s/pdif for 1,2'")
    print("         \t5 -'Meter Digital In, use s/pdif for 1,2'")
    print("         \t6 -'Analog Out, use aes s/pdif 1,2'")
    print("         \t7 -'Digital Out, use aes s/pdif 1,2'")
    print("")
    print("  --set_meter=METER\tset meters to METER")
    print("        \tMETER is a number 0-3:")
    print("         \t0 -'Analog In'")
    print("         \t1 -'Digital In'")
    print("         \t2 -'Analog Out'")
    print("         \t3 -'Digital Out'")
    print("")
    print("  --set_dig1=DIGIN\tset digital inputs 1,2 to DIGIN")
    print("        \tDIGIN is 0 or 1:")
    print("         \t0 -'AES 1,2'")
    print("         \t1 -'S/PDIF'")
    print("")
    print("  --sci,--set_channel_input_gain=GAIN CHANNEL [CHANNEL...] \n\t \
        set analog INPUT gain of CHANNELs to GAIN")
    print("        \tGAIN is an integer value between -95 and +32")
    print("        \tCHANNELs are integer values between 1-8:")
    print("         \t\t1-8 -'INPUT Channels 1-8'")
    print("")
    print("  --sco,--set_channel_output_gain=GAIN CHANNEL [CHANNEL...] \n\t \
        set analog OUTPUT gain of CHANNELs to GAIN")
    print("        \tGAIN - an integer value between -95 and +32")
    print("        \tCHANNELs - integer values between 1-8:")
    print("         \t\t1-8 -'OUTPUT Channels 1-8'")
    print("")
    print("  --set_gain=PRESET")
    print("\t set analog INPUTS and OUTPUT to recommended PRESET")
    print("        \tPRESET is an integer value +4 or -10:")
    print("         \t'+4' -Sets all INPUTS to -8dB and all OUTPUTS to +1dB")
    print("         \t'-10' -Sets all INPUTS to +4dB and all OUTPUTS to -11dB")
    print("\nGET OPTIONS:")
    print("**GET OPTIONS ARE FOR RS232 MODEL UNITS ONLY**")
    print("  -g,--get_all\tget all values from lucid")
    print("  --get_sync\tget source of clock sync")
    print("  --get_analog\tget source of analog output")
    print("  --get_aes\tget source of aes output")
    print("  --get_opt\tget source of optical output")
    print("  --get_meter\tget source of meters")
    print("  --get_dig1\tget output of digital channels 1&2")
    print("  --get_gain\tget analog gain settings")

def error_exit(errormsg="Fatal Error"):
    """Exit the command-line with 1 and a passed-in errormsg
    """
    logging.critical(errormsg)
    #usage()
    print("--help will provide usage options")
    sys.exit(1)

def get_aes_src(lucid):
    """Call lucid8824.get_aes_source, exit on failure"""
    print("AES Source:\t%s" % (lucid.get_aes_source() or
                               sys.exit("failed talking to Lucid")))


def get_analog_src(lucid):
    """Call lucid8824.get_analog_source, exit on failure"""
    print("Analog Source:\t%s" % (lucid.get_analog_source() or
                                  sys.exit("failed talking to 8824")))


def get_opt_src(lucid):
    """Call lucid8824.get_opt_source, exit on failure"""
    print("Optical Source:\t%s" % (lucid.get_opt_source() or
                                   sys.exit("failed talking to 8824")))


def get_meter(lucid):
    """Call lucid8824.get_meter, exit on failure"""
    print("Meter:\t\t%s" % (lucid.get_meter() or
                            sys.exit("failed talking to 8824")))


def get_sync(lucid):
    """Call lucid8824.get_sync, exit on failure"""
    print("Sync:\t\t%s" % (lucid.get_sync_source()))

def get_dig1(lucid):
    """Call lucid8824.get_dig1, exit on failure"""
    print("Dig Input 1,2:\t%s" % (lucid.get_dig1() or
                                  sys.exit("failed talking to 8824")))


def get_gain(lucid):
    """Call lucid8824.get_get, exit on failure"""
    gainlist = lucid.get_gain() or sys.exit("failed talking to Lucid")
    print("Analog Gain:")
    print(' '*40)
    print('***************************************')
    print('Recommended: +4dBu: IN -8 dB OUT  +1 dB')
    print('            -10dBV: IN +4 dB OUT -11 dB')
    print('***************************************')
    print(' '*40)
    for i in range(1, 9):
        print('\tChannel {0:1d}: IN {1:3s}dB OUT  {2:3s}dB'
              .format(i, gainlist[i-1], gainlist[i+7]))


def set_gain_channels(lucid, gain=-99, channel_list=[]):
    """Set the channels in `channel_list` to `gain`
    Write these values to the 8824
    """
    gain = int(gain)
    logging.info("set_gain_channels: value of gain is %s" % gain)
    if (type(channel_list) != list):
            logging.critical("list received: %s" %
                             type(channel_list))
            sys.exit(1)
    elif (gain < -95 or
            gain > 32):
            logging.critical("bad argument to set_gain_channels: %s" %
                             gain)
            sys.exit(1)
    for i in channel_list:
        if int(i) < 0 or int(i) > 15:
            logging.critical("channel index out of range")
            sys.exit(1)

    logging.info("set_gain_channels: calling get_gain...")
    lucid.get_gain() or sys.exit("failed talking to 8824")

    logging.info("set_gain_channels: updating gainlist")
    for ch in channel_list:
        logging.info("set_gain_channels: setting channel %s to %s" %
                     (ch, gain))
        lucid.update_channel_in_gainlist(gain, ch)

    logging.info("set_gain_channels: writing gainlist to Lucid")
    lucid.write_gainlist_to_lucid()


def get_all(lucid):
    """Get all possible values from the 8824 by calling each
    function sequentially
    """
    get_sync(lucid)
    get_meter(lucid)
    get_analog_src(lucid)
    get_aes_src(lucid)
    get_opt_src(lucid)
    get_dig1(lucid)
    get_gain(lucid)


def set_sync(lucid, syncval):
    """Call lucid8824.set_sync_source"""
    lucid.set_sync_source(syncval)
    if lucid.siface in lucid.DEVICES['rs232']:
        get_sync(lucid)

def set_meter_and_dig1(lucid, meterval):
    """Call lucid8824.set_meter"""
    lucid.set_meter_and_dig1(meterval)
    if lucid.siface in lucid.DEVICES['rs232']:
        get_meter(lucid)
        get_dig1(lucid)

def set_meter(lucid, meterval):
    """Call lucid8824.set_meter"""
    if lucid.siface in lucid.DEVICES['rs232']:
        lucid.set_meter(meterval)
        get_meter(lucid)


def set_dig1(lucid, srcval):
    """Call lucid8824.set_dig1"""
    lucid.set_dig1(srcval)
    if lucid.siface in lucid.DEVICES['rs232']:
        get_dig1(lucid)


def set_analog_src(lucid, srcval):
    """Call lucid8824.set_analog_src"""
    lucid.set_analog_src(srcval)


def set_opt_src(lucid, srcval):
    """Call lucid8824.set_opt_src"""
    lucid.set_opt_src(srcval)
    if lucid.siface in lucid.DEVICES['rs232']:
        get_opt_src(lucid)
                
def set_aes_src(lucid, srcval):
    """Call lucid8824.set_aes_src"""
    lucid.set_aes_src(srcval)


def main():
    """Run the glucid8824 Command Line Interface"""

    lucid = False
    serialif = '/dev/ttyUSB0'
    device_id = '00'
    
    banner()

    try:
        opts, args = getopt.getopt(sys.argv[1:],
                                   "hvmgd:D:i:",
                                   [
                                       "help",
                                       "verbose",
                                       "get_all",
                                       "get_aes",
                                       "set_aes=",
                                       "get_analog",
                                       "set_analog=",
                                       "get_opt",
                                       "set_opt=",
                                       "get_sync",
                                       "set_sync=",
                                       "get_meter",
                                       "set_meter=",
                                       "get_gain",
                                       "set_gain=",
                                       "set_channel_input_gain=",
                                       "sci=",
                                       "set_channel_output_gain=",
                                       "sco=",
                                       "get_dig1",
                                       "set_dig1=",
                                       "set_meter_and_dig1=",
                                       "device_id="
                                ]
        )
    except getopt.GetoptError as err:
        print(err)
        usage()
        sys.exit(2)

    verbose = False
    # if not default serial interface
    serialif = '/dev/ttyUSB0'
    # first parse for verbose option
    for o, a in opts:
        if o in ("-v", "--verbose"):
            verbose = True
            logging.basicConfig(level=logging.INFO)

    if verbose:
        logging.basicConfig(level=logging.INFO)

    # read interface from configfile
    glucidconf = configparser.ConfigParser()
    #glucidconf.read(CONFIGFILE)
    glucidconf.read(os.path.join(os.path.expanduser('~'), CONFIGFILE))

    if 'Device' in glucidconf['DEFAULT']:
        serialif = glucidconf['DEFAULT']['Device']
        logging.info("Read default %s from configfile" % serialif)
    if 'DEVICE_ID' in glucidconf['DEFAULT']:
        device_id = glucidconf['DEFAULT']['DEVICE_ID']
        logging.info("Read device_id %s from configfile" % device_id)

    newconfig = open(os.path.join(os.path.expanduser('~'), CONFIGFILE), 'w')
            
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(0)
        elif o in ("-d"):
            # set device to user command line argument
            serialif = a
            logging.info("setting serialif to %s" % serialif)
        elif o in ("-D"):
            # set device to user command line argument
            # AND write to config file
            serialif = a
            logging.info("new default device %s" % serialif)
            glucidconf.set('DEFAULT','Device',serialif)
            newconfig.close()
        elif o in ("-i"):
            device_id = '{:02}'.format(int(a))
            logging.info("new default device_id %s" % device_id)
        elif o in ("-I"):
            device_id = '{:02}'.format(int(a))
            logging.info("new default device_id %s" % device_id)
            glucidconf.set('DEFAULT','DEVICE_ID',device_id)

    glucidconf.write(newconfig)
    logging.info("Wrote config to  %s" % os.path.join(os.path.expanduser('~'), CONFIGFILE))

    lucid = Glucid8824(siface=serialif,LucidID=device_id)

    if not lucid.connect():
        sys.exit("Failed to open connection using %s \n" %
                 lucid.get_iface())

    print("Using %s to connect to lucid ID %s\n" %
          (lucid.get_iface(), lucid.get_instanceid()))

    for o, a in opts:
        if o in ("-h", "--help", "-v", "--verbose", "-d"):
            continue
        elif o in ("-g", "--get_all"):
            logging.info("get all")
            get_all(lucid)
            sys.exit(0)
        elif o in ("--get_aes"):
            logging.info("get aes src")
            get_aes_src(lucid)
        elif o in ("--set_aes"):
            if int(a) >= 0 and int(a) <= 1:
                logging.info("setting aes to %s" % a)
                set_aes_src(lucid, int(a))
                get_aes_src(lucid)
            else:
                logging.critical("bad argument to set_aes_src")
                usage()
                sys.exit(1)
        elif o in ("--get_analog"):
            logging.info("get analog src")
            get_analog_src(lucid)
        elif o in ("--set_analog"):
            if int(a) >= 0 and int(a) <= 1:
                logging.info("setting analog to %s" % a)
                set_analog_src(lucid, int(a))
                get_analog_src(lucid)
            else:
                logging.critical("bad argument to set_analog_src")
                usage()
                sys.exit(1)
        elif o in ("--get_opt"):
            logging.info("get aes src")
            get_opt_src(lucid)
        elif o in ("--set_opt"):
            if int(a) >= 0 and int(a) <= 1:
                logging.info("setting optical to %s" % a)
                set_opt_src(lucid, int(a))
            else:
                logging.critical("bad argument to set_opt_src")
                usage()
                sys.exit(1)
        elif o in ("--get_sync"):
            logging.info("get sync")
            get_sync(lucid)
        elif o in ("--set_sync"):
            if int(a) >= 0 and int(a) <= 7:
                logging.info("setting sync to %s" % a)
                set_sync(lucid, int(a))
            else:
                logging.critical("bad argument to set_sync")
                usage()
                sys.exit(1)
        elif o in ("--get_meter"):
            logging.info("get meter")
            get_meter(lucid)
        elif o in ("--set_meter"):
            if int(a) >= 0 and int(a) <= 3:
                logging.info("setting meter to %s" % a)
                set_meter(lucid, int(a))
            else:
                logging.critical("bad argument to set_meter")
                usage()
                sys.exit(1)
        elif o in ("--set_meter_and_dig1"):
            if int(a) >= 0 and int(a) <= 7:
                logging.info("setting meter_dig1 to %s" % a)
                set_meter_and_dig1(lucid, int(a))
            else:
                logging.critical("bad argument to set_meter_and_dig1")
                usage()
                sys.exit(1)
        elif o in ("--get_dig1"):
            logging.info("get dig1")
            get_dig1(lucid)
        elif o in ("--set_dig1"):
            if int(a) >= 0 and int(a) <= 1:
                logging.info("setting dig1 to %s" % a)
                set_dig1(lucid, int(a))
            else:
                logging.critical("bad argument to set_dig1")
                usage()
                sys.exit(1)
        elif o in ("--get_gain"):
            logging.info("get gain")
            get_gain(lucid)
        elif o in ('--set_gain'):
            logging.info("set gain")
            if (int(a) != 4 and int(a) != -10):
                logging.critical("bad argument to set_gain")
                sys.exit(1)
            if int(a) == 4:
                set_gain_channels(lucid, -8, [0, 1, 2, 3, 4, 5, 6, 7])
                set_gain_channels(lucid, 1, [8, 9, 10, 11, 12, 13, 14, 15])
            elif int(a) == -10:
                set_gain_channels(lucid, 4, [0, 1, 2, 3, 4, 5, 6, 7])
                set_gain_channels(lucid, -11, [8, 9, 10, 11, 12, 13, 14, 15])
                get_gain(lucid)
        elif o in ('--sci', '--set_channel_input_gain',
                   '--sco', '--set_channel_output_gain'):
            # make sure gain value is valid
            if (
                int(a) < -95
                or int(a) > 32
                or len(args) == 0
               ):
                    logging.critical("Bad Argument: %s, %" % (o, a))
                    sys.exit(1)
            # make sure user chose channels to apply gain to
            elif(len(args) == 0):
                    logging.critical("No Channels Specified")
                    sys.exit(1)
            # make sure channels are valid
            for arg in args:
                if int(arg) < 1 or int(arg) > 8:
                    logging.critical("Invalid Channel: %s" % arg)
                    sys.exit(1)

            if o in ('--sci', '--set_channel_input_gain'):
                # Input Channels Index at Zero: 0-15
                set_gain_channels(lucid, int(a),
                                  [int(i) - 1 for i in args])
            elif o in ('--sco', '--set_channel_output_gain'):
                # Output Channels are 7-15
                set_gain_channels(lucid, int(a),
                                  [int(i) + 7 for i in args])

            get_gain(lucid)
        else:
            usage_light()
            sys.exit(0)


if __name__ == "__main__":
    main()



from setuptools import setup
# To use a consistent encoding
from codecs import open
from os import path

with open('README.rst') as f:
    long_desc = f.read()

setup(name='glucid',
      version='0.4.0',
      description='Configure the Lucid 8824 AD/DA Audio Interface via \
             a Serial Connection',
      url='http://github.com/danmechanic/glucid',
      author='Daniel R Mechanic',
      author_email='dan.mechanic@gmail.com',
      license='GPL V3',
      zip_safe=False,
#      scripts=['bin/glucid','bin/xglucid'],
      entry_points={  # Optional
          'console_scripts': [
              'glucid=glucid.glucid_cli:main',
              'xglucid=glucid.xglucid:main'
          ],
      },
      long_description=long_desc,
      keywords='lucid 8824 audio converter',
      packages=['glucid'],
#      py_modules=['glucid.glucid'],
      python_requires=">=3",
      package_dir={ 'glucid8824' : 'glucid',
                    'xglucid' : 'glucid',
                    'Glucid8824_UI' : 'glucid',
                    'Glucid8824' : 'glucid',
                    'xglucidUIWidgets' : 'glucid',
      },
      long_description_content_type='text/x-rst',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
          'Topic :: Multimedia :: Sound/Audio :: Conversion',
          'Topic :: Multimedia :: Sound/Audio :: Capture/Recording',
      ],
      project_urls={
          'Author': 'http://www.danmechanic.com/',
          'Source': 'https://github.com/danmechanic/glucid/',
      },
      install_requires=[
          'PyQt5>=5.9',
	  'PySerial',
      ]
      )
